# -*- coding: utf-8 ; test-case-name: bridgedb.test.test_schedule -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Nick Mathewson
#           Isis Lovecruft <isis@torproject.org> 0xa3adb67a2cdb8b35
# :copyright: (c) 2007-2015, The Tor Project, Inc.
#             (c) 2014-2015, Isis Lovecruft
# :license: see LICENSE for licensing information

"""This module implements functions for dividing time into chunks."""

import calendar

from datetime import datetime

from zope import interface
from zope.interface import implements
from zope.interface import Attribute


#: The known time intervals (or *periods*) for dividing time by.
KNOWN_INTERVALS = ["second", "minute", "hour", "day", "week", "month"]


class UnknownInterval(ValueError):
    """Raised if an interval isn't one of the :data:`KNOWN_INTERVALS`."""


def toUnixSeconds(timestruct):
    """Convert a datetime struct to a Unix timestamp in seconds.

    :param timestruct: A ``datetime.datetime`` object to convert into a
        timestamp in Unix Era seconds.
    :rtype: int
    """
    return calendar.timegm(timestruct)

def fromUnixSeconds(timestamp):
    """Convert a Unix timestamp to a datetime struct.

    :param int timestamp: A timestamp in Unix Era seconds.
    :rtype: :type:`datetime.datetime`
    """
    return datetime.fromtimestamp(timestamp)


class ISchedule(interface.Interface):
    """A ``Interface`` specification for a Schedule."""

    intervalPeriod = Attribute(
        "The type of period which this Schedule's intervals will rotate by.")
    intervalCount = Attribute(
        "Number of **intervalPeriod**s before rotation to the next interval")

    def intervalStart(when=None):
        """Get the start time of the interval that contains **when**."""

    def getInterval(when=None):
        """Get the interval which includes an arbitrary **when**."""

    def nextIntervalStarts(when=None):
        """Get the start of the interval after the one containing **when**."""


class Unscheduled(object):
    """A base ``Schedule`` that has only one period that contains all time."""

    implements(ISchedule)

    def __init__(self, period=None, count=None):
        """Create a schedule for dividing time into intervals.

        :param str period: One of the periods in :data:`KNOWN_INTERVALS`.
        :param int count: The number of **period**s in an interval.
        """
        self.intervalCount = count
        self.intervalPeriod = period

    def intervalStart(self, when=0):
        """Get the start time of the interval that contains **when**.

        :param int when: The time which we're trying to find the corresponding
            interval for.
        :rtype: int
        :returns: The Unix epoch timestamp for the start time of the interval
            that contains **when**.
        """
        return toUnixSeconds(datetime.min.timetuple())

    def getInterval(self, when=0):
        """Get the interval that contains the time **when**.

        .. note: We explicitly ignore the ``when`` parameter in this
            implementation because if something is Unscheduled then
            all timestamps should reside within the same period.

        :param int when: The time which we're trying to find the corresponding
            interval for.
        :rtype: str
        :returns: A timestamp in the form ``YEAR-MONTH[-DAY[-HOUR]]``. It's
            specificity depends on what type of interval we're using. For
            example, if using ``"month"``, the return value would be something
            like ``"2013-12"``.

        """
        return fromUnixSeconds(0).strftime('%04Y-%02m-%02d %02H:%02M:%02S')

    def nextIntervalStarts(self, when=0):
        """Return the start time of the interval starting _after_ when.

        :rtype: int
        :returns: Return the Y10K bug.
        """
        return toUnixSeconds(datetime.max.timetuple())


class ScheduledInterval(Unscheduled):
    """An class that splits time into periods, based on seconds, minutes,
    hours, days, weeks, or months.

    :ivar str intervalPeriod: One of the :data:`KNOWN_INTERVALS`.
    :ivar int intervalCount: The number of times **intervalPeriod** should be
        repeated within an interval.
    """
    implements(ISchedule)

    def __init__(self, period=None, count=None):
        """Create a schedule for dividing time into intervals.

        :param str period: One of the periods in :data:`KNOWN_INTERVALS`.
        :param int count: The number of **period**s in an interval.
        """
        super(ScheduledInterval, self).__init__(period, count)
        self._setIntervalCount(count)
        self._setIntervalPeriod(period)

    def _setIntervalCount(self, count=None):
        """Set our :ivar:`intervalCount`.

        .. attention:: This method should be called _before_
            :meth:`_setIntervalPeriod`, because the latter may change the
            count, if it decides to change the period (for example, to
            simplify things by changing weeks into days).

        :param int count: The number of times the :ivar:`intervalPeriod`
            should be repeated during the interval. Defaults to ``1``.
        :raises UnknownInterval: if the specified **count** was invalid.
        """
        try:
            if not count > 0:
                count = 1
            count = int(count)
        except (TypeError, ValueError):
            raise UnknownInterval("%s.intervalCount: %r ist not an integer."
                                  % (self.__class__.__name__, count))
        self.intervalCount = count

    def _setIntervalPeriod(self, period=None):
        """Set our :ivar:`intervalPeriod`.

        :param str period: One of the :data:`KNOWN_INTERVALS`, or its
            plural. Defaults to ``'hour'``.
        :raises UnknownInterval: if the specified **period** is unknown.
        """
        if not period:
            period = 'hour'
        try:
            period = period.lower()
            # Depluralise the period if necessary, i.e., "months" -> "month".
            if period.endswith('s'):
                period = period[:-1]

            if not period in KNOWN_INTERVALS:
                raise ValueError
        except (TypeError, AttributeError, ValueError):
            raise UnknownInterval("%s doesn't know about the %r interval type."
                                  % (self.__class__.__name__, period))
        self.intervalPeriod = period

        if period == 'week':
            self.intervalPeriod = 'day'
            self.intervalCount *= 7

    def intervalStart(self, when=0):
        """Get the start time of the interval that contains **when**.

        :param int when: The time which we're trying to determine the start of
            interval that contains it. This should be given in Unix seconds,
            for example, taken from :func:`calendar.timegm`.
        :rtype: int
        :returns: The Unix epoch timestamp for the start time of the interval
            that contains **when**.
        """
        if self.intervalPeriod == 'month':
            # For months, we always start at the beginning of the month.
            date = fromUnixSeconds(when)
            months = (date.year * 12) + (date.month - 1)
            months -= (months % self.intervalCount)
            month = months % 12 + 1
            return toUnixSeconds((months // 12, month, 1, 0, 0, 0))
        elif self.intervalPeriod == 'day':
            # For days, we start at the beginning of a day.
            when -= when % (86400 * self.intervalCount)
            return when
        elif self.intervalPeriod == 'hour':
            # For hours, we start at the beginning of an hour.
            when -= when % (3600 * self.intervalCount)
            return when
        elif self.intervalPeriod == 'minute':
            when -= when % (60 * self.intervalCount)
            return when
        elif self.intervalPeriod == 'second':
            when -= when % self.intervalCount
            return when

    def getInterval(self, when=0):
        """Get the interval that contains the time **when**.

        >>> import calendar
        >>> from bridgedb.schedule import ScheduledInterval
        >>> sched = ScheduledInterval('month', 1)
        >>> when = calendar.timegm((2007, 12, 12, 0, 0, 0))
        >>> sched.getInterval(when)
        '2007-12'
        >>> then = calendar.timegm((2014, 05, 13, 20, 25, 13))
        >>> sched.getInterval(then)
        '2014-05'

        :param int when: The time which we're trying to find the corresponding
            interval for. Given in Unix seconds, for example, taken from
            :func:`calendar.timegm`.
        :rtype: str
        :returns: A timestamp in the form ``YEAR-MONTH[-DAY[-HOUR]]``. It's
            specificity depends on what type of interval we're using. For
            example, if using ``"month"``, the return value would be something
            like ``"2013-12"``.
        """
        date = fromUnixSeconds(self.intervalStart(when))

        fstr = "%04Y-%02m"
        if self.intervalPeriod != 'month':
            fstr += "-%02d"
            if self.intervalPeriod != 'day':
                fstr += " %02H"
                if self.intervalPeriod != 'hour':
                    fstr += ":%02M"
                    if self.intervalPeriod == 'minute':
                        fstr += ":%02S"

        return date.strftime(fstr)

    def nextIntervalStarts(self, when=0):
        """Return the start time of the interval starting _after_ when.

        :returns: The Unix epoch timestamp for the start time of the interval
            that contains **when**.
        """
        seconds = self.intervalStart(when)

        if self.intervalPeriod == 'month':
            date = fromUnixSeconds(seconds)
            year = date.year
            months = date.month + self.intervalCount
            if months > 12:
                year = date.year + 1
                months = months - 12
            return toUnixSeconds((year, months, 1, 0, 0, 0))
        elif self.intervalPeriod == 'day':
            return seconds + (86400 * self.intervalCount)
        elif self.intervalPeriod == 'hour':
            return seconds + (3600 * self.intervalCount)
        elif self.intervalPeriod == 'minute':
            return seconds + (60 * self.intervalCount)
        elif self.intervalPeriod == 'second':
            return seconds + self.intervalCount



# -*- coding: utf-8 -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
#           please also see AUTHORS file
# :copyright: (c) 2013, Isis Lovecruft
#             (c) 2007-2013, The Tor Project, Inc.
#             (c) 2007-2013, all entities within the AUTHORS file
# :license: 3-Clause BSD, see LICENSE for licensing information

"""Unittests for the :mod:`bridgedb.parse.addr` module.
"""

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import ipaddr
import random

from twisted.python import log
from twisted.trial import unittest

from bridgedb.parse import addr


IP4LinkLocal = "169.254.0.0"
IP6LinkLocal = "fe80::1234"
IP4Loopback = "127.0.0.0"
IP4Localhost = "127.0.0.1"
IP6Localhost = "::1"
IP4LimitedBroadcast = "255.255.255.0"
IP4Multicast_224 = "224.0.0.1"
IP4Multicast_239 = "239.0.0.1"
IP4Unspecified = "0.0.0.0"
IP6Unspecified = "::"
IP4DefaultRoute = "0.0.0.0"
IP6DefaultRoute = "::"
IP4ReservedRFC1918_10 = "10.0.0.0"
IP4ReservedRFC1918_172_16 = "172.16.0.0"
IP4ReservedRFC1918_192_168 = "192.168.0.0"
IP4ReservedRFC1700 = "240.0.0.0"
IP6UniqueLocal = "fc00::"
IP6SiteLocal = "fec0::"


class CanonicalizeEmailDomainTests(unittest.TestCase):
    """Unittests for :func:`bridgedb.parse.addr.canonicalizeEmailDomain`."""

    def test_nonDict(self):
        """Using a non-dict domainmap as a parameter to
        canonicalizeEmailDomain() should log an AttributeError and then raise
        an UnsupportedDomain error.
        """
        domainmap = 'example.com'
        domain = 'fubar.com'
        self.assertRaises(addr.UnsupportedDomain,
                          addr.canonicalizeEmailDomain,
                          domain, domainmap)

    def test_notPermitted(self):
        """A domain not in the domainmap of allowed domains should raise an
        UnsupportedDomain error.
        """
        domainmap = {'foo.example.com': 'example.com'}
        domain = 'bar.example.com'
        self.assertRaises(addr.UnsupportedDomain,
                          addr.canonicalizeEmailDomain,
                          domain, domainmap)

    def test_permitted(self):
        """A domain in the domainmap of allowed domains should return the
        canonical domain.
        """
        domainmap = {'foo.example.com': 'example.com'}
        domain = 'foo.example.com'
        canonical = addr.canonicalizeEmailDomain(domain, domainmap)
        self.assertEquals(canonical, 'example.com')


class ExtractEmailAddressTests(unittest.TestCase):
    """Unittests for :func:`bridgedb.parse.addr.extractEmailAddress`."""

    def test_23(self):
        """The email address int(23) should raise a BadEmail error."""
        self.assertRaises(addr.BadEmail,
                          addr.extractEmailAddress,
                          int(23))

    def test_lessThanChars(self):
        """The email address 'Alice <alice@riseup.net>' should return
        ('alice', 'riseup.net').
        """
        local, domain = addr.extractEmailAddress('Alice <alice@riseup.net>')
        self.assertEqual(local, 'alice')
        self.assertEqual(domain, 'riseup.net')

    def test_extraLessThanChars(self):
        """The email address 'Mallory <mal<lory@riseup.net>' should return
        ('lory', 'riseup.net')
        """
        local, domain = addr.extractEmailAddress('Mallory <mal<lory@riseup.net>')
        self.assertEqual(local, 'lory')
        self.assertEqual(domain, 'riseup.net')

    def test_extraLessAndGreaterThanChars(self):
        """The email address 'Mallory <mal><>>lory@riseup.net>' should raise a
        BadEmail error.
        """
        self.assertRaises(addr.BadEmail,
                          addr.extractEmailAddress,
                          'Mallory <mal><>>lory@riseup.net>')

    def test_extraAppendedEmailAddress(self):
        """The email address 'Mallory <mallory@riseup.net><mallory@gmail.com>'
        should use the last address.
        """
        local, domain = addr.extractEmailAddress(
            'Mallory <mallory@riseup.net><mallory@gmail.com>')
        self.assertEqual(local, 'mallory')
        self.assertEqual(domain, 'gmail.com')


class NormalizeEmailTests(unittest.TestCase):
    """Unittests for :func:`bridgedb.parse.addr.normalizeEmail`."""

    def test_permitted(self):
        """A valid email address from a permitted domain should return
        unchanged.
        """
        domainrules = {}
        domainmap = {'foo.example.com': 'example.com'}
        emailaddr = 'alice@foo.example.com'
        normalized = addr.normalizeEmail(emailaddr, domainmap, domainrules)
        self.assertEqual(emailaddr, normalized)

    def test_notPermitted(self):
        """A valid email address from a non-permitted domain should raise an
        UnsupportedDomain error.
        """
        domainrules = {}
        domainmap = {'bar.example.com': 'example.com'}
        emailaddr = 'Alice <alice@foo.example.com>'
        self.assertRaises(addr.UnsupportedDomain,
                          addr.normalizeEmail,
                          emailaddr, domainmap, domainrules)

    def test_ignoreDots(self):
        """A valid email address with a '.' should remove the '.' if
        'ignore_dots' is in domainrules.
        """
        domainrules = {'example.com': 'ignore_dots'}
        domainmap = {'foo.example.com': 'example.com'}
        emailaddr = 'alice.bridges@foo.example.com'
        normalized = addr.normalizeEmail(emailaddr, domainmap, domainrules)
        self.assertEqual('alicebridges@foo.example.com', normalized)

    def test_ignorePlus(self):
        """A valid email address with a '+' and some extra stuff, from a
        permitted domain, should remove the '+' stuff if 'ignore_plus' is
        enabled.
        """
        domainrules = {}
        domainmap = {'foo.example.com': 'example.com'}
        emailaddr = 'alice+bridges@foo.example.com'
        normalized = addr.normalizeEmail(emailaddr, domainmap, domainrules)
        self.assertEqual('alice@foo.example.com', normalized)

    def test_dontIgnorePlus(self):
        """A valid email address with a '+' and some extra stuff, from a
        permitted domain, should return unchanged if 'ignore_plus' is disabled.
        """
        domainrules = {}
        domainmap = {'foo.example.com': 'example.com'}
        emailaddr = 'alice+bridges@foo.example.com'
        normalized = addr.normalizeEmail(emailaddr, domainmap, domainrules,
                                         ignorePlus=False)
        self.assertEqual(emailaddr, normalized)


class ParseAddrIsIPAddressTests(unittest.TestCase):
    """Unittests for :func:`bridgedb.parse.addr.isIPAddress`.

    .. note:: All of the ``test_isIPAddress_IP*`` methods in this class should
       get ``False`` as the **result** returned from :func:`addr.isIPAddress`,
       because all the ``IP*`` constants defined above are invalid addresses
       according to :func:`addr.isValidIP`.
    """

    def test_isIPAddress_randomIP4(self):
        """Test :func:`addr.isIPAddress` with a random IPv4 address.

        This test asserts that the returned IP address is not None (because
        the IP being tested is random, it *could* randomly be an invalid IP
        address and thus :func:`~bridgdb.addr.isIPAddress` would return
        ``False``).
        """
        randomAddress = ipaddr.IPv4Address(random.getrandbits(32))
        result = addr.isIPAddress(randomAddress)
        log.msg("Got addr.isIPAddress() result for random IPv4 address %r: %s"
                % (randomAddress, result))
        self.assertTrue(result is not None)

    def test_isIPAddress_randomIP6(self):
        """Test :func:`addr.isIPAddress` with a random IPv6 address.

        This test asserts that the returned IP address is not None (because
        the IP being tested is random, it *could* randomly be an invalid IP
        address and thus :func:`~bridgdb.addr.isIPAddress` would return
        ``False``).
        """
        randomAddress = ipaddr.IPv6Address(random.getrandbits(128))
        result = addr.isIPAddress(randomAddress)
        log.msg("Got addr.isIPAddress() result for random IPv6 address %r: %s"
                % (randomAddress, result))
        self.assertTrue(result is not None)

    def runTestForAddr(self, testAddress):
        """Test :func:`addr.isIPAddress` with the specified ``testAddress``.

        :param str testAddress: A string which specifies either an IPv4 or
                                IPv6 address to test.
        """
        result = addr.isIPAddress(testAddress)
        log.msg("addr.isIPAddress(%r) => %s" % (testAddress, result))
        self.assertTrue(result is not None,
                        "Got a None for testAddress: %r" % testAddress)
        self.assertFalse(isinstance(result, basestring),
                        "Expected %r result from isIPAddress(%r): %r %r"
                        % (bool, testAddress, result, type(result)))

    def test_isIPAddress_IP4LinkLocal(self):
        """Test :func:`addr.isIPAddress` with a link local IPv4 address."""
        self.runTestForAddr(IP4LinkLocal)

    def test_isIPAddress_IP6LinkLocal(self):
        """Test :func:`addr.isIPAddress` with a link local IPv6 address."""
        self.runTestForAddr(IP6LinkLocal)

    def test_isIPAddress_IP4Loopback(self):
        """Test :func:`addr.isIPAddress` with the loopback IPv4 address."""
        self.runTestForAddr(IP4Loopback)

    def test_isIPAddress_IP4Localhost(self):
        """Test :func:`addr.isIPAddress` with a localhost IPv4 address."""
        self.runTestForAddr(IP4Localhost)

    def test_isIPAddress_IP6LinkLocal(self):
        """Test :func:`addr.isIPAddress` with a localhost IPv6 address."""
        self.runTestForAddr(IP6Localhost)

    def test_isIPAddress_IP4LimitedBroadcast(self):
        """Test :func:`addr.isIPAddress` with a limited broadcast IPv4
        address.
        """
        self.runTestForAddr(IP4LimitedBroadcast)

    def test_isIPAddress_IP4Multicast_224(self):
        """Test :func:`addr.isIPAddress` with a multicast IPv4 address."""
        self.runTestForAddr(IP4Multicast_224)

    def test_isIPAddress_IP4Multicast_239(self):
        """Test :func:`addr.isIPAddress` with a multicast IPv4 address."""
        self.runTestForAddr(IP4Multicast_239)

    def test_isIPAddress_IP4Unspecified(self):
        """Test :func:`addr.isIPAddress` with an unspecified IPv4 address."""
        self.runTestForAddr(IP4Unspecified)

    def test_isIPAddress_IP6Unspecified(self):
        """Test :func:`addr.isIPAddress` with an unspecified IPv6 address."""
        self.runTestForAddr(IP6Unspecified)

    def test_isIPAddress_IP4DefaultRoute(self):
        """Test :func:`addr.isIPAddress` with a default route IPv4 address."""
        self.runTestForAddr(IP4DefaultRoute)

    def test_isIPAddress_IP6DefaultRoute(self):
        """Test :func:`addr.isIPAddress` with a default route IPv6 address."""
        self.runTestForAddr(IP6DefaultRoute)

    def test_isIPAddress_IP4ReservedRFC1918_10(self):
        """Test :func:`addr.isIPAddress` with a reserved IPv4 address."""
        self.runTestForAddr(IP4ReservedRFC1918_10)

    def test_isIPAddress_IP4ReservedRFC1918_172_16(self):
        """Test :func:`addr.isIPAddress` with a reserved IPv4 address."""
        self.runTestForAddr(IP4ReservedRFC1918_172_16)

    def test_isIPAddress_IP4ReservedRFC1918_192_168(self):
        """Test :func:`addr.isIPAddress` with a reserved IPv4 address."""
        self.runTestForAddr(IP4ReservedRFC1918_192_168)

    def test_isIPAddress_IP4ReservedRFC1700(self):
        """Test :func:`addr.isIPAddress` with a :rfc:`1700` reserved IPv4
        address.
        """
        self.runTestForAddr(IP4ReservedRFC1700)

    def test_isIPAddress_IP6UniqueLocal(self):
        """Test :func:`addr.isIPAddress` with an unique local IPv6 address."""
        self.runTestForAddr(IP6UniqueLocal)

    def test_isIPAddress_IP6SiteLocal(self):
        """Test :func:`addr.isIPAddress` with a site local IPv6 address."""
        self.runTestForAddr(IP6SiteLocal)

    def test_isIPAddress_withNonIP(self):
        """Test :func:`addr.isIPAddress` with non-IP input."""
        self.runTestForAddr('not an ip address')

    def test_filehandle(self):
        """Test :func:`addr.isIPAddress` with a file handle for input.

        Try to raise a non- :exc:`~exceptions.ValueError` exception in
        :func:`addr.isIPAddress`.
        """
        fh = open('{0}-filehandle'.format(self.__class__.__name__), 'wb')
        self.runTestForAddr(fh)

    def test_returnUncompressedIP(self):
        """Test returning a :class:`ipaddr.IPAddress`."""
        testAddress = '86.59.30.40'
        result = addr.isIPAddress(testAddress, compressed=False)
        log.msg("addr.isIPAddress(%r, compressed=False) => %r"
                % (testAddress, result))
        self.assertTrue(
            isinstance(result, ipaddr.IPv4Address),
            "Expected %r result from isIPAddress(%r, compressed=False): %r %r"
            % (ipaddr.IPv4Address, testAddress, result, type(result)))

    def test_unicode(self):
        """Test with unicode input."""
        self.runTestForAddr("↙↓↘←↔→↖↑↗↙↓↘←↔→↖↑↗")


class ParseAddrIsIPv4Tests(unittest.TestCase):
    """Unittests for :func:`bridgedb.parse.addr.isIPv4`."""

    def runTestForIPv4(self, testAddress):
        """Test :func:`addr.isIPv4` with the specified IPv4 **testAddress**.

        This test asserts that the returned value is ``True``.

        :param str testAddress: A string which specifies the IPv4 address to
           test, which should cause :func:`addr.isIPv4` to return ``True``.
        """
        result = addr.isIPv4(testAddress)
        log.msg("addr.isIPv4(%r) => %s" % (testAddress, result))
        self.assertTrue(isinstance(result, bool),
                        "addr.isIPv4() should be boolean: %r" % type(result))
        self.assertTrue(result,
                        "addr.isIPv4(%r) should be True!" % testAddress)

    def runTestForIPv6(self, testAddress):
        """Test :func:`addr.isIPv4` with the specified IPv6 **testAddress**.

        This test asserts that the returned value is ``False``.

        :param str testAddress: A string which specifies the IPv6 address to
            test, which should cause :func:`addr.isIPv4` to return ``False``.
        """
        result = addr.isIPv4(testAddress)
        log.msg("addr.isIPv4(%r) => %s" % (testAddress, result))
        self.assertTrue(isinstance(result, bool),
                        "addr.isIPv4() should be boolean: %r" % type(result))
        self.assertFalse(result,
                         "addr.isIPv4(%r) should be False!" % testAddress)

    def test_isIPv4_randomIP4(self):
        """Test :func:`addr.isIPv4` with a random IPv4 address.

        This test asserts that the returned value is a :obj:`bool`. Because
        the IP being tested is random, it *could* randomly be an invalid IP
        address and thus :func:`~bridgdb.addr.isIPv4` would return ``False``).
        """
        randomAddr = ipaddr.IPv4Address(random.getrandbits(32)).compressed
        log.msg("Testing randomly generated IPv4 address: %s" % randomAddr)
        result = addr.isIPv4(randomAddr)
        self.assertTrue(isinstance(result, bool),
                        "addr.isIPv4() should be boolean: %r" % type(result))

    def test_isIPv4_randomIP6(self):
        """Test :func:`addr.isIPv4` with a random IPv6 address."""
        randomAddr = ipaddr.IPv6Address(random.getrandbits(128)).compressed
        log.msg("Testing randomly generated IPv6 address: %s" % randomAddr)
        self.runTestForIPv6(randomAddr)

    def test_isIPv4_IP4LinkLocal(self):
        """Test :func:`addr.isIPv4` with a link local IPv4 address."""
        self.runTestForIPv4(IP4LinkLocal)

    def test_isIPv4_IP6LinkLocal(self):
        """Test :func:`addr.isIPv4` with a link local IPv6 address."""
        self.runTestForIPv6(IP6LinkLocal)

    def test_isIPv4_IP4Loopback(self):
        """Test :func:`addr.isIPv4` with the loopback IPv4 address."""
        self.runTestForIPv4(IP4Loopback)

    def test_isIPv4_IP4Localhost(self):
        """Test :func:`addr.isIPv4` with a localhost IPv4 address."""
        self.runTestForIPv4(IP4Localhost)

    def test_isIPv4_IP6Localhost(self):
        """Test :func:`addr.isIPv4` with a localhost IPv6 address."""
        self.runTestForIPv6(IP6Localhost)

    def test_isIPv4_IP4LimitedBroadcast(self):
        """Test :func:`addr.isIPv4` with a multicast IPv4 address."""
        self.runTestForIPv4(IP4LimitedBroadcast)

    def test_isIPv4_IP4Multicast_224(self):
        """Test :func:`addr.isIPv4` with a multicast IPv4 address."""
        self.runTestForIPv4(IP4Multicast_224)

    def test_isIPv4_IP4Multicast_239(self):
        """Test :func:`addr.isIPv4` with a multicast IPv4 address."""
        self.runTestForIPv4(IP4Multicast_239)

    def test_isIPv4_IP4Unspecified(self):
        """Test :func:`addr.isIPv4` with an unspecified IPv4 address."""
        self.runTestForIPv4(IP4Unspecified)

    def test_isIPv4_IP6Unspecified(self):
        """Test :func:`addr.isIPv4` with an unspecified IPv6 address."""
        self.runTestForIPv6(IP6Unspecified)

    def test_isIPv4_IP4DefaultRoute(self):
        """Test :func:`addr.isIPv4` with a default route IPv4 address."""
        self.runTestForIPv4(IP4DefaultRoute)

    def test_isIPv4_IP6DefaultRoute(self):
        """Test :func:`addr.isIPv4` with a default route IPv6 address."""
        self.runTestForIPv6(IP6DefaultRoute)

    def test_isIPv4_IP4ReservedRFC1918_10(self):
        """Test :func:`addr.isIPv4` with a reserved IPv4 address."""
        self.runTestForIPv4(IP4ReservedRFC1918_10)

    def test_isIPv4_IP4ReservedRFC1918_172_16(self):
        """Test :func:`addr.isIPv4` with a reserved IPv4 address."""
        self.runTestForIPv4(IP4ReservedRFC1918_172_16)

    def test_isIPv4_IP4ReservedRFC1918_192_168(self):
        """Test :func:`addr.isIPv4` with a reserved IPv4 address."""
        self.runTestForIPv4(IP4ReservedRFC1918_192_168)

    def test_isIPv4_IP4ReservedRFC1700(self):
        """Test :func:`addr.isIPv4` with a :rfc:`1700` reserved IPv4
        address.
        """
        self.runTestForIPv4(IP4ReservedRFC1700)

    def test_isIPv4_IP6UniqueLocal(self):
        """Test :func:`addr.isIPv4` with an unique local IPv6 address."""
        self.runTestForIPv6(IP6UniqueLocal)

    def test_isIPv4_IP6SiteLocal(self):
        """Test :func:`addr.isIPv4` with a site local IPv6 address."""
        self.runTestForIPv6(IP6SiteLocal)

    def test_isIPv4_withValidIPv4(self):
        """Test :func:`addr.isIPv4` with a valid IPv4 address."""
        self.runTestForIPv4('38.229.72.2')

    def test_isIPv4_withValidIPv4_2(self):
        """Test :func:`addr.isIPv4` with a valid IPv4 address."""
        self.runTestForIPv4('15.15.15.15')

    def test_isIPv4_withValidIPv4_3(self):
        """Test :func:`addr.isIPv4` with a valid IPv4 address."""
        self.runTestForIPv4('93.95.227.222')

    def test_isIPv4_withValidIPv6(self):
        """Test :func:`addr.isIPv4` with a valid IPv6 address."""
        self.runTestForIPv6("2a00:1450:4001:808::1010")

    def test_isIPv4_withNonIP(self):
        """Test :func:`addr.isIPv4` with non-IP input."""
        self.runTestForIPv6('not an ip address')


class ParseAddrIsIPv6Tests(unittest.TestCase):
    """Unittests for :func:`bridgedb.parse.addr.isIPv6`.

    .. note:: All of the ``test_isIPv6_IP*`` methods in this class should get
       ``False`` as their **result** value returned from :func:`addr.isIPv6`,
       because all of the ``IP*`` constants defined above are invalid
       according to :func:`addr.isValidIP`.
    """

    def runTestForIPv4(self, testAddress):
        """Test :func:`addr.isIPv6` with the specified IPv4 **testAddress**.

        This test asserts that the returned value is ``False``.

        :param str testAddress: A string which specifies the IPv4 address to
           test, which should cause :func:`addr.isIPv6` to return ``False``.
        """
        result = addr.isIPv6(testAddress)
        log.msg("addr.isIPv6(%r) => %s" % (testAddress, result))
        self.assertTrue(isinstance(result, bool),
                        "addr.isIPv6() should be boolean: %r" % type(result))
        self.assertFalse(result,
                        "addr.isIPv6(%r) should be False!" % testAddress)

    def runTestForIPv6(self, testAddress):
        """Test :func:`addr.isIPv6` with the specified IPv6 **testAddress**.

        This test asserts that the returned value is ``True``.

        Random addresses should *not* be tested with this function, because
        :func:`~addr.isIPv6` uses :func:`~addr.isValidIP` internally, and will
        return False if the IP is invalid.

        :param str testAddress: A string which specifies the IPv6 address to
            test, which should cause :func:`addr.isIPv6` to return ``True``.
        """
        result = addr.isIPv6(testAddress)
        log.msg("addr.isIPv6(%r) => %s" % (testAddress, result))
        self.assertTrue(isinstance(result, bool),
                        "addr.isIPv6() should be boolean: %r" % type(result))
        self.assertTrue(result,
                        "addr.isIPv6(%r) should be True!" % testAddress)

    def test_isIPv6_randomIP4(self):
        """Test :func:`addr.isIPv6` with a random IPv4 address."""
        randomAddr = ipaddr.IPv4Address(random.getrandbits(32)).compressed
        log.msg("Testing randomly generated IPv4 address: %s" % randomAddr)
        self.runTestForIPv4(randomAddr)

    def test_isIPv6_randomIP6(self):
        """Test :func:`addr.isIPv6` with a random IPv6 address.

        This test asserts that the returned IP address is a :obj:`bool`
        (because the IP being tested is random, it *could* randomly be an
        invalid IP address and thus :func:`~bridgdb.addr.isIPv6` would return
        ``False``).
        """
        randomAddr = ipaddr.IPv6Address(random.getrandbits(128)).compressed
        log.msg("Testing randomly generated IPv6 address: %s" % randomAddr)
        result = addr.isIPv6(randomAddr)
        self.assertTrue(isinstance(result, bool),
                        "addr.isIPv6() should be boolean: %r" % type(result))

    def test_isIPv6_IP4LinkLocal(self):
        """Test :func:`addr.isIPv6` with a link local IPv4 address.

        :meth:`runTestForIPv4` is used because this address is invalid
        according to :func:`addr.isValidIP`; therefore, the result from
        :func:`addr.isIPv6` should be ``False``.
        """
        self.runTestForIPv4(IP4LinkLocal)

    def test_isIPv6_IP6LinkLocal(self):
        """Test :func:`addr.isIPv6` with a link local IPv6 address."""
        self.runTestForIPv6(IP6LinkLocal)

    def test_isIPv6_IP4Loopback(self):
        """Test :func:`addr.isIPv6` with the loopback IPv4 address."""
        self.runTestForIPv4(IP4Loopback)

    def test_isIPv6_IP4Localhost(self):
        """Test :func:`addr.isIPv6` with a localhost IPv4 address."""
        self.runTestForIPv4(IP4Localhost)

    def test_isIPv6_IP6Localhost(self):
        """Test :func:`addr.isIPv6` with a localhost IPv6 address."""
        self.runTestForIPv6(IP6Localhost)

    def test_isIPv6_IP4LimitedBroadcast(self):
        """Test :func:`addr.isIPv6` with a multicast IPv4 address."""
        self.runTestForIPv4(IP4LimitedBroadcast)

    def test_isIPv6_IP4Multicast_224(self):
        """Test :func:`addr.isIPv6` with a multicast IPv4 address."""
        self.runTestForIPv4(IP4Multicast_224)

    def test_isIPv6_IP4Multicast_239(self):
        """Test :func:`addr.isIPv6` with a multicast IPv4 address."""
        self.runTestForIPv4(IP4Multicast_239)

    def test_isIPv6_IP4Unspecified(self):
        """Test :func:`addr.isIPv6` with an unspecified IPv4 address."""
        self.runTestForIPv4(IP4Unspecified)

    def test_isIPv6_IP6Unspecified(self):
        """Test :func:`addr.isIPv6` with an unspecified IPv6 address."""
        self.runTestForIPv6(IP6Unspecified)

    def test_isIPv6_IP4DefaultRoute(self):
        """Test :func:`addr.isIPv6` with a default route IPv4 address."""
        self.runTestForIPv4(IP4DefaultRoute)

    def test_isIPv6_IP6DefaultRoute(self):
        """Test :func:`addr.isIPv6` with a default route IPv6 address."""
        self.runTestForIPv6(IP6DefaultRoute)

    def test_isIPv6_IP4ReservedRFC1918_10(self):
        """Test :func:`addr.isIPv6` with a reserved IPv4 address."""
        self.runTestForIPv4(IP4ReservedRFC1918_10)

    def test_isIPv6_IP4ReservedRFC1918_172_16(self):
        """Test :func:`addr.isIPv6` with a reserved IPv4 address."""
        self.runTestForIPv4(IP4ReservedRFC1918_172_16)

    def test_isIPv6_IP4ReservedRFC1918_192_168(self):
        """Test :func:`addr.isIPv6` with a reserved IPv4 address."""
        self.runTestForIPv4(IP4ReservedRFC1918_192_168)

    def test_isIPv6_IP4ReservedRFC1700(self):
        """Test :func:`addr.isIPv6` with a :rfc:`1700` reserved IPv4
        address.
        """
        self.runTestForIPv4(IP4ReservedRFC1700)

    def test_isIPv6_IP6UniqueLocal(self):
        """Test :func:`addr.isIPv6` with an unique local IPv6 address."""
        self.runTestForIPv6(IP6UniqueLocal)

    def test_isIPv6_IP6SiteLocal(self):
        """Test :func:`addr.isIPv6` with a site local IPv6 address."""
        self.runTestForIPv6(IP6SiteLocal)

    def test_isIPv6_withValidIPv4(self):
        """Test :func:`addr.isIPv6` with a valid IPv4 address."""
        self.runTestForIPv4('38.229.72.2')

    def test_isIPv6_withValidIPv4_2(self):
        """Test :func:`addr.isIPv6` with a valid IPv4 address."""
        self.runTestForIPv4('15.15.15.15')

    def test_isIPv6_withValidIPv4_3(self):
        """Test :func:`addr.isIPv6` with a valid IPv4 address."""
        self.runTestForIPv4('93.95.227.222')

    def test_isIPv6_withValidIPv6(self):
        """Test :func:`addr.isIPv6` with a valid IPv6 address."""
        self.runTestForIPv6("2a00:1450:4001:808::1010")

    def test_isIPv6_withNonIP(self):
        """Test :func:`addr.isIPv6` with non-IP input."""
        self.runTestForIPv4('not an ip address')


class PortListTest(unittest.TestCase):
    """Unittests for :class:`bridgedb.parse.addr.PortList`."""

    def getRandomPort(self):
        """Get a port in the range [1, 65535] inclusive.

        :rtype: int
        :returns: A random port number.
        """
        return random.randint(1, 65535)

    def test_tooFewPorts(self):
        """Create a :class:`addr.PortList` with no ports at all."""
        portList = addr.PortList()
        self.assertEqual(len(portList), 0)

    def test_tooManyPorts(self):
        """Create a :class:`addr.PortList` with more than the maximum
        allowed ports, as given in ``PortList.PORTSPEC_LEN``.

        We don't currently do anything to deal with a PortList having too many
        ports.
        """
        tooMany = addr.PortList.PORTSPEC_LEN + 1
        ports = [self.getRandomPort() for x in xrange(tooMany)]
        log.msg("Testing addr.PortList(%s))"
                % ', '.join([type('')(port) for port in ports]).strip(', '))
        portList = addr.PortList(*ports)
        self.assertEqual(len(portList), tooMany)

    def test_invalidPortNumber(self):
        """Test creating a :class:`addr.PortList` with an invalid port.

        Should raise an InvalidPort error.
        """
        self.assertRaises(addr.InvalidPort, addr.PortList, 66666, 6666)

    def test_contains(self):
        """Test creating a :class:`addr.PortList` with valid ports.

        Then check that ``__contains__`` works properly.
        """
        ports = (443, 9001, 9030)
        portList = addr.PortList(*ports)
        self.assertIn(443, portList)

    def test_iter(self):
        """Test creating a :class:`addr.PortList` with valid ports.

        Then check that ``__iter__`` works properly.
        """
        ports = (443, 9001, 9030)
        portList = addr.PortList(*ports)
        iterator = iter(portList)
        for x in xrange(len(ports)):
            self.assertIn(iterator.next(), portList)

    def test_str(self):
        """Test creating a :class:`addr.PortList` with valid ports.

        Then check that ``__str__`` works properly.
        """
        ports = (443, 9001, 9030)
        portList = addr.PortList(*ports)
        self.assertTrue(isinstance(str(portList), basestring))
        for port in ports:
            self.assertIn(str(port), str(portList))

    def test_getitem_shouldContain(self):
        """Test ``__getitem__`` with a port number in the PortList."""
        ports = (443, 9001, 9030)
        portList = addr.PortList(*ports)
        self.assertEqual(portList.__getitem__(0), 9001)

    def test_getitem_shouldNotContain(self):
        """Test ``__getitem__`` with a port number not in the PortList."""
        ports = (443, 9001, 9030)
        portList = addr.PortList(*ports)
        self.assertRaises(IndexError, portList.__getitem__, 555)

    def test_getitem_string(self):
        """Test ``__getitem__`` with a string."""
        ports = (443, 9001, 9030)
        portList = addr.PortList(*ports)
        self.assertRaises(TypeError, portList.__getitem__, '443')

    def test_getitem_long(self):
        """Test ``__getitem__`` with a string."""
        ports = (443, 9001, 9030)
        portList = addr.PortList(*ports)
        self.assertEqual(portList.__getitem__(long(0)), 9001)

    def test_mixedArgs(self):
        """Create a :class:`addr.PortList` with mixed type parameters."""
        firstList = addr.PortList('1111,2222,3333')
        portList = addr.PortList(443, "9001,9030, 9050", firstList)
        self.assertTrue(portList)

    def test_invalidStringArgs(self):
        """Create a :class:`addr.PortList` with mixed type parameters."""
        self.assertRaises(addr.InvalidPort,
                          addr.PortList, '1111, 666666, 3333')



# -*- coding: utf-8 ; test-case-name: bridgedb.test.test_txrecaptcha -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
# :copyright: (c) 2013-2015, Isis Lovecruft
#             (c) 2007-2015, The Tor Project, Inc.
# :license: 3-Clause BSD, see LICENSE for licensing information

"""Twisted-based reCAPTCHA client.

This client *always* uses TLS with strict hostname checking, unlike the
official Google Python recaptcha-client_, which is harcoded_ to use plaintext
HTTP.

Small portions of this code were taken from the official Google Python
recaptcha-client_ module, version 1.0.6.  Those portions are
:class:`RecaptchaResponse`, :data:`API_SERVER`, They total 5 lines of code,
which are copyright the authors of the recaptcha-client_ package.

.. _hardcoded: https://code.google.com/p/recaptcha/source/browse/trunk/recaptcha-plugins/python/recaptcha/client/captcha.py#76
.. _recaptcha-client: https://pypi.python.org/pypi/recaptcha-client/1.0.6
"""

import logging
import urllib

from OpenSSL.crypto import FILETYPE_PEM
from OpenSSL.crypto import load_certificate

from twisted import version as _twistedversion
from twisted.internet import defer
from twisted.internet import protocol
from twisted.internet import reactor
from twisted.python import failure
from twisted.python.versions import Version
from twisted.web import client
from twisted.web.http_headers import Headers
from twisted.web.iweb import IBodyProducer

from zope.interface import implements

from bridgedb.crypto import SSLVerifyingContextFactory

#: This was taken from  recaptcha.client.captcha.API_SSL_SERVER.
API_SSL_SERVER = API_SERVER = "https://www.google.com/recaptcha/api"
API_SSL_VERIFY_URL = "%s/verify" % API_SSL_SERVER

#: (type: `OpenSSL.crypto.X509`) Only trust certificate for the reCAPTCHA
#: :data:`API_SSL_SERVER` which were signed by the Google Internet Authority CA.
GOOGLE_INTERNET_AUTHORITY_CA_CERT = load_certificate(FILETYPE_PEM, bytes("""\
-----BEGIN CERTIFICATE-----
MIICsDCCAhmgAwIBAgIDFXfhMA0GCSqGSIb3DQEBBQUAME4xCzAJBgNVBAYTAlVT
MRAwDgYDVQQKEwdFcXVpZmF4MS0wKwYDVQQLEyRFcXVpZmF4IFNlY3VyZSBDZXJ0
aWZpY2F0ZSBBdXRob3JpdHkwHhcNMTIxMjEyMTU1ODUwWhcNMTMxMjMxMTU1ODUw
WjBGMQswCQYDVQQGEwJVUzETMBEGA1UEChMKR29vZ2xlIEluYzEiMCAGA1UEAxMZ
R29vZ2xlIEludGVybmV0IEF1dGhvcml0eTCBnzANBgkqhkiG9w0BAQEFAAOBjQAw
gYkCgYEAye23pIucV+eEPkB9hPSP0XFjU5nneXQUr0SZMyCSjXvlKAy6rWxJfoNf
NFlOCnowzdDXxFdF7dWq1nMmzq0yE7jXDx07393cCDaob1FEm8rWIFJztyaHNWrb
qeXUWaUr/GcZOfqTGBhs3t0lig4zFEfC7wFQeeT9adGnwKziV28CAwEAAaOBozCB
oDAfBgNVHSMEGDAWgBRI5mj5K9KylddH2CMgEE8zmJCf1DAdBgNVHQ4EFgQUv8Aw
6/VDET5nup6R+/xq2uNrEiQwEgYDVR0TAQH/BAgwBgEB/wIBADAOBgNVHQ8BAf8E
BAMCAQYwOgYDVR0fBDMwMTAvoC2gK4YpaHR0cDovL2NybC5nZW90cnVzdC5jb20v
Y3Jscy9zZWN1cmVjYS5jcmwwDQYJKoZIhvcNAQEFBQADgYEAvprjecFG+iJsxzEF
ZUNgujFQodUovxOWZshcnDW7fZ7mTlk3zpeVJrGPZzhaDhvuJjIfKqHweFB7gwB+
ARlIjNvrPq86fpVg0NOTawALkSqOUMl3MynBQO+spR7EHcRbADQ/JemfTEh2Ycfl
vZqhEFBfurZkX0eTANq98ZvVfpg=
-----END CERTIFICATE-----"""))

# `t.w.client.HTTPConnectionPool` isn't available in Twisted-12.0.0
# (see ticket #11219: https://bugs.torproject.org/11219):
_connectionPoolAvailable = _twistedversion >= Version('twisted', 12, 1, 0)
if _connectionPoolAvailable:
    logging.info("Using HTTPConnectionPool for reCaptcha API server.")
    _pool = client.HTTPConnectionPool(reactor, persistent=False)
    _pool.maxPersistentPerHost = 5
    _pool.cachedConnectionTimeout = 30
    _agent = client.Agent(reactor, pool=_pool)
else:
    logging.warn("Twisted-%s is too old for HTTPConnectionPool! Disabling..."
                 % _twistedversion.short())
    _pool = None
    _agent = client.Agent(reactor)


# Twisted>=14.0.0 changed the way in which hostname verification works.
if _twistedversion >= Version('twisted', 14, 0, 0):
    from twisted.internet._sslverify import OpenSSLCertificateAuthorities

    class RecaptchaOpenSSLCertificateAuthorities(OpenSSLCertificateAuthorities):
        """The trusted CAs for connecting to reCAPTCHA servers."""
        #: A list of `OpenSSL.crypto.X509` objects.
        caCerts = [GOOGLE_INTERNET_AUTHORITY_CA_CERT,]
        def __init__(self):
            super(RecaptchaOpenSSLCertificateAuthorities, self).__init__(self.caCerts)

    class RecaptchaPolicyForHTTPS(client.BrowserLikePolicyForHTTPS):
        _trustRoot = RecaptchaOpenSSLCertificateAuthorities()
        def __init__(self):
            super(RecaptchaPolicyForHTTPS, self).__init__(trustRoot=self._trustRoot)


def _setAgent(agent):
    """Set the global :attr:`agent`.

    :param agent: An :api:`twisted.web.client.Agent` for issuing requests.
    """
    global _agent
    _agent = agent

def _getAgent(reactor=reactor, url=API_SSL_VERIFY_URL, connectTimeout=30,
              **kwargs):
    """Create a :api:`twisted.web.client.Agent` which will verify the
    certificate chain and hostname for the given **url**.

    :param reactor: A provider of the
        :api:`twisted.internet.interface.IReactorTCP` interface.
    :param str url: The full URL which will be requested with the
        ``Agent``. (default: :attr:`API_SSL_VERIFY_URL`)
    :param pool: An :api:`twisted.web.client.HTTPConnectionPool`
        instance. (default: :attr:`_pool`)
    :type connectTimeout: None or int
    :param connectTimeout: If not ``None``, the timeout passed to
        :api:`twisted.internet.reactor.connectTCP` or
        :api:`twisted.internet.reactor.connectSSL` for specifying the
        connection timeout. (default: ``30``)
    """
    # Twisted>=14.0.0 changed the way in which hostname verification works.
    if _twistedversion >= Version('twisted', 14, 0, 0):
        contextFactory = RecaptchaPolicyForHTTPS()
    else:
        contextFactory = SSLVerifyingContextFactory(url)

    if _connectionPoolAvailable:
        return client.Agent(reactor,
                            contextFactory=contextFactory,
                            connectTimeout=connectTimeout,
                            pool=_pool,
                            **kwargs)
    else:
        return client.Agent(reactor,
                            contextFactory=contextFactory,
                            connectTimeout=connectTimeout,
                            **kwargs)

_setAgent(_getAgent())


class RecaptchaResponseError(ValueError):
    """There was an error with the reCaptcha API server's response."""


class RecaptchaResponse(object):
    """Taken from recaptcha.client.captcha.`RecaptchaResponse`_.
    .. RecaptchaResponse: https://code.google.com/p/recaptcha/source/browse/trunk/recaptcha-plugins/python/recaptcha/client/captcha.py#7
    """
    def __init__(self, is_valid, error_code=None):
        self.is_valid = is_valid
        self.error_code = error_code


class RecaptchaResponseProtocol(protocol.Protocol):
    """HTML parser which creates a :class:`RecaptchaResponse` from the body of
    the reCaptcha API server's response.
    """
    def __init__(self, finished):
        """Create a protocol for creating :class:`RecaptchaResponse`s.

        :type finished: :api:`~twisted.internet.defer.Deferred`
        :param finished: A deferred which will have its ``callback()`` called
             with a :class:`RecaptchaResponse`.
        """
        self.finished = finished
        self.remaining = 1024 * 10
        self.response = ''

    def dataReceived(self, data):
        """Called when some data is received from the connection."""
        if self.remaining:
            received = data[:self.remaining]
            self.response += received
            self.remaining -= len(received)

    def connectionLost(self, reason):
        """Called when the connection was closed.

        :type reason: :api:`twisted.python.failure.Failure`
        :param reason: A string explaning why the connection was closed,
            wrapped in a ``Failure`` instance.

        :raises: A :api:`twisted.internet.error.ConnectError` if the 
        """
        valid = False
        error = reason.getErrorMessage()
        try:
            (valid, error) = self.response.strip().split('\n', 1)
        except ValueError:
            error = "Couldn't parse response from reCaptcha API server"

        valid = bool(valid == "true")
        result = RecaptchaResponse(is_valid=valid, error_code=error)
        logging.debug(
            "ReCaptcha API server response: %s(is_valid=%s, error_code=%s)"
            % (result.__class__.__name__, valid, error))
        self.finished.callback(result)


class _BodyProducer(object):
    """I write a string into the HTML body of an open request."""
    implements(IBodyProducer)

    def __init__(self, body):
        self.body = body
        self.length = len(body)

    def startProducing(self, consumer):
        """Start writing the HTML body."""
        consumer.write(self.body)
        return defer.succeed(None)

    def pauseProducing(self):
        pass

    def stopProducing(self):
        pass

    def resumeProducing(self):
        pass


def _cbRequest(response):
    """Callback for a :api:`twisted.web.client.Agent.request` which delivers
    the result to a :class:`RecaptchaResponseProtocol`.

    :returns: A :api:`~twisted.internet.defer.Deferred` which will callback
    with a ``recaptcha.RecaptchaResponse`` for the request.
    """
    finished = defer.Deferred()
    response.deliverBody(RecaptchaResponseProtocol(finished))
    return finished

def _ebRequest(fail):
    """Errback for a :api:`twisted.web.client.Agent.request`.

    :param fail: A :api:`twisted.python.failure.Failure` which occurred during
        the request.
    """
    logging.debug("txrecaptcha._ebRequest() called with %r" % fail)
    error = fail.getErrorMessage() or "possible problem in _ebRequest()"
    return RecaptchaResponse(is_valid=False, error_code=error)

def _encodeIfNecessary(string):
    """Encode unicode objects in utf-8 if necessary."""
    if isinstance(string, unicode):
        return string.encode('utf-8')
    return string

def submit(recaptcha_challenge_field, recaptcha_response_field,
           private_key, remoteip, agent=_agent):
    """Submits a reCaptcha request for verification. This function is a patched
    version of the ``recaptcha.client.captcha.submit()`` function in
    reCaptcha's Python API.

    It does two things differently:
        1. It uses Twisted for everything.
        2. It uses SSL/TLS for everything.

    This function returns a :api:`~twisted.internet.defer.Deferred`. If you
    need a ``recaptcha.client.captcha.RecaptchaResponse`` to be returned, use
    the :func:`submit` function, which is an ``@inlineCallbacks`` wrapper for
    this function.

    :param str recaptcha_challenge_field: The value of the HTTP POST
        ``recaptcha_challenge_field`` argument from the form.
    :param recaptcha_response_field: The value of the HTTP POST
        ``recaptcha_response_field`` argument from the form.
    :param private_key: The reCAPTCHA API private key.
    :param remoteip: An IP address to give to the reCaptcha API server.
    :returns: A :api:`~twisted.internet.defer.Deferred` which will callback
        with a ``recaptcha.RecaptchaResponse`` for the request.
    """
    if not (recaptcha_response_field and len(recaptcha_response_field) and
            recaptcha_challenge_field and len(recaptcha_challenge_field)):
        d = defer.Deferred()
        d.addBoth(_ebRequest)  # We want `is_valid=False`
        d.errback(failure.Failure(ValueError('incorrect-captcha-sol')))
        return d

    params = urllib.urlencode({
        'privatekey': _encodeIfNecessary(private_key),
        'remoteip':   _encodeIfNecessary(remoteip),
        'challenge':  _encodeIfNecessary(recaptcha_challenge_field),
        'response':   _encodeIfNecessary(recaptcha_response_field)})
    body = _BodyProducer(params)
    headers = Headers({"Content-type": ["application/x-www-form-urlencoded"],
                       "User-agent": ["reCAPTCHA Python"]})
    d = agent.request('POST', API_SSL_VERIFY_URL, headers, body)
    d.addCallbacks(_cbRequest, _ebRequest)
    return d



# -*- coding: utf-8 -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
#           please also see AUTHORS file
# :copyright: (c) 2007-2013, The Tor Project, Inc.
#             (c) 2007-2013, all entities within the AUTHORS file
# :license: 3-clause BSD, see included LICENSE for information

"""_langs.py - Storage for information on installed language support."""

RTL_LANGS = ('ar', 'he', 'fa', 'gu_IN', 'ku')


def get_langs():
    """Return a list of two-letter country codes of translations which were
    installed (if we've already been installed).
    """
    return supported


#: This list will be rewritten by :func:`get_supported_langs` in setup.py at
#: install time, so that the :attr:`bridgedb.__langs__` will hold a list of
#: two-letter country codes for languages which were installed.
supported = set(['ar', 'az', 'bg', 'ca', 'cs', 'cy', 'da', 'de', 'el', 'en', 'en_GB', 'en_US', 'eo', 'es', 'es_CL', 'eu', 'fa', 'fi', 'fr', 'fr_CA', 'gl', 'he', 'hr_HR', 'hu', 'id', 'it', 'ja', 'km', 'kn', 'ko', 'lv', 'mk', 'ms_MY', 'nb', 'nl', 'pl', 'pt', 'pt_BR', 'ro', 'ru', 'si_LK', 'sk', 'sk_SK', 'sl_SI', 'sq', 'sv', 'ta', 'th', 'tr', 'uk', 'zh_CN', 'zh_TW'])



# BridgeDB by Nick Mathewson.
# Copyright (c) 2007-2012, The Tor Project, Inc.
# See LICENSE for licensing information 

from ipaddr import IPv6Address, IPv4Address
import logging

funcs = {}

def filterAssignBridgesToRing(hmac, numRings, assignedRing):
    #XXX: ruleset should have a key unique to this function
    # ruleset ensures that the same 
    logging.debug("Creating a filter for assigning bridges to hashrings...")
    ruleset = frozenset([hmac, numRings, assignedRing]) 
    logging.debug("Filter created: %s" % ruleset)

    try: 
        return funcs[ruleset]
    except KeyError:
        def _assignBridgesToRing(bridge):
            digest = hmac(bridge.getID())
            pos = long( digest[:8], 16 )
            which = pos % numRings

            if which == assignedRing:
                return True
            else:
                logging.debug(("Bridge %s has calculated assignment %d; not "
                               "in correct ring %d.")
                              % (bridge, which, assignedRing))
                return False

        _assignBridgesToRing.__name__ = ("filterAssignBridgesToRing(%s, %s, %s)"
                                         % (hmac, numRings, assignedRing))
        setattr(_assignBridgesToRing, "description", "ring=%d" % assignedRing)
        funcs[ruleset] = _assignBridgesToRing
        return _assignBridgesToRing

def filterBridgesByRules(rules):
    ruleset = frozenset(rules)
    try: 
        return funcs[ruleset] 
    except KeyError:
        def g(x):
            r = [f(x) for f in rules]
            if False in r: return False
            return True
        setattr(g, "description", " ".join([getattr(f,'description','') for f in rules]))
        funcs[ruleset] = g
        return g  

def filterBridgesByIP4(bridge):
    try:
        if IPv4Address(bridge.address): return True
    except ValueError:
        pass

    for address, port, version in bridge.allVanillaAddresses:
        if version == 4:
            return True
    return False
setattr(filterBridgesByIP4, "description", "ip=4")

def filterBridgesByIP6(bridge):
    try:
        if IPv6Address(bridge.address): return True
    except ValueError:
        pass

    for address, port, version in bridge.allVanillaAddresses:
        if version == 6:
            return True
    return False
setattr(filterBridgesByIP6, "description", "ip=6")

def filterBridgesByTransport(methodname, addressClass=None):
    if not ((addressClass is IPv4Address) or (addressClass is IPv6Address)):
        addressClass = IPv4Address

    ruleset = frozenset([methodname, addressClass])
    try:
        return funcs[ruleset]
    except KeyError:

        def _filterByTransport(bridge):
            for transport in bridge.transports:
                if isinstance(transport.address, addressClass):
                    # ignore method name case
                    if transport.methodname.lower() == methodname.lower():
                        return True
                    else:
                        logging.debug(("Transport methodname '%s' doesn't match "
                                       "requested methodname: '%s'.")
                                      % (transport.methodname, methodname))
                else:
                    logging.debug(("Transport %s has incorrect address version "
                                   "(%s).") % (transport, addressClass))
            return False

        _filterByTransport.__name__ = ("filterBridgesByTransport(%s,%s)"
                                       % (methodname, addressClass))
        setattr(_filterByTransport, "description", "transport=%s" % methodname)
        funcs[ruleset] = _filterByTransport
        return _filterByTransport

def filterBridgesByNotBlockedIn(countryCode, addressClass=None, methodname=None):
    """ if at least one address:port of the selected addressClass and
    (optional) transport type is not blocked in countryCode, return True
    """
    # default to IPv4 if not specified
    if addressClass is None: addressClass = IPv4Address
    assert (addressClass) in (IPv4Address, IPv6Address)
    ruleset = frozenset([countryCode, addressClass, methodname])
    try:
        return funcs[ruleset]
    except KeyError:
        def f(bridge):
            if bridge.isBlocked(countryCode, addressClass, methodname):
                if addressClass is IPv4Address: ac = "IPv4"
                else: ac = "IPv6"
                logmsg = "Removing %s from set of results for country"
                logmsg += " '%s' with address class %s and transport %s"
                logging.debug(logmsg % ( bridge.fingerprint, countryCode, ac,
                    methodname))
                return False
            return True # not blocked
        f.__name__ = "filterBridgesNotBlockedIn(%s,%s,%s)" % \
                (countryCode,methodname,addressClass)
        funcs[ruleset] = f
        return f



# -*- coding: utf-8 ; test-case-name: bridgedb.test.test_bridges -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: please see the AUTHORS file for attributions
# :copyright: (c) 2013-2015, Isis Lovecruft
#             (c) 2007-2015, The Tor Project, Inc.
# :license: see LICENSE for licensing information

"""Classes for manipulating and storing Bridges and their attributes."""



# -*- coding: utf-8 -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
#           please also see AUTHORS file
# :copyright: (c) 2014-2015, Isis Lovecruft
#             (c) 2007-2015, The Tor Project, Inc.
#             (c) 2007-2015, all entities within the AUTHORS file
# :license: see LICENSE for licensing information

"""Unittests for the :mod:`bridgedb.util` module."""

from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import logging
import os

from twisted.mail.smtp import Address
from twisted.trial import unittest

from bridgedb import util


class MiscLoggingUtilTests(unittest.TestCase):
    """Unittests for miscellaneous logging functions in :mod:`bridgedb.util`."""

    def test_getLogHandlers(self):
        """util._getLogHandlers() should return ['rotating', 'console'] if
        both stderr and logfile logging are enabled.
        """
        logHandlers = util._getLogHandlers()
        self.assertIsInstance(logHandlers, list)
        self.assertEqual(len(logHandlers), 2)

    def test_getLogHandlers_disableStderr(self):
        """util._getLogHandlers() should return ['rotating'] if stderr logging
        is disabled.
        """
        logHandlers = util._getLogHandlers(logToStderr=False)
        self.assertIsInstance(logHandlers, list)
        self.assertEqual(len(logHandlers), 1)
        self.assertTrue('console' not in logHandlers)

    def test_getLogHandlers_disable_logfile(self):
        """util._getLogHandlers() should return ['console'] if stderr logging
        is disabled.
        """
        logHandlers = util._getLogHandlers(logToFile=False)
        self.assertIsInstance(logHandlers, list)
        self.assertEqual(len(logHandlers), 1)
        self.assertTrue('rotating' not in logHandlers)

    def test_getRotatingFileHandler(self):
        """_getRotatingFileHandler() should create a file with 0600
        permissions (os.ST_WRITE | os.ST_APPEND).
        """
        filename = str(self.id()) + '.log'
        logHandler = util._getRotatingFileHandler(filename)
        self.assertTrue(os.path.isfile(filename))
        self.assertEqual(os.stat_result(os.stat(filename)).st_mode, 33152)
        self.assertIsInstance(logHandler(),
                              util.logging.handlers.RotatingFileHandler)

    def test_configureLogging(self):
        """Configure logging should be callable without borking anything."""
        from bridgedb.persistent import Conf
        util.configureLogging(Conf())
        util.logging.info("BridgeDB's email address: bridges@torproject.org")


class LevenshteinDistanceTests(unittest.TestCase):
    """Unittests for `bridgedb.util.levenshteinDistance."""

    def test_levenshteinDistance_blank_blank(self):
        """The Levenshtein Distance between '' and '' should be 0."""
        distance = util.levenshteinDistance('', '')
        self.assertEqual(distance, 0)

    def test_levenshteinDistance_cat_cat(self):
        """The Levenshtein Distance between 'cat' and 'cat' should be 0."""
        distance = util.levenshteinDistance('cat', 'cat')
        self.assertEqual(distance, 0)

    def test_levenshteinDistance_bat_cat(self):
        """The Levenshtein Distance between 'bat' and 'cat' should be 1."""
        distance = util.levenshteinDistance('bat', 'cat')
        self.assertEqual(distance, 1)

    def test_levenshteinDistance_bar_cat(self):
        """The Levenshtein Distance between 'bar' and 'cat' should be 2."""
        distance = util.levenshteinDistance('bar', 'cat')
        self.assertEqual(distance, 2)

    def test_levenshteinDistance_bridgedb_doge(self):
        """The Levenshtein Distance between 'bridgedb' and 'doge' should be 6."""
        distance = util.levenshteinDistance('bridgedb', 'doge')
        self.assertEqual(distance, 6)

    def test_levenshteinDistance_feidanchaoren0043_feidanchaoren0011(self):
        """The Levenshtein Distance between the usernames in
        'feidanchaoren0043@gmail.com' and 'feidanchaoren0011@gmail.com' should
        be less than an EMAIL_FUZZY_MATCH parameter.
        """
        email1 = Address('feidanchaoren0043@gmail.com')
        email2 = Address('feidanchaoren0011@gmail.com')
        # Fuzzy match if the Levenshtein Distance is less than or equal to:
        fuzzyMatch = 4
        distance = util.levenshteinDistance(email1.local, email2.local)
        self.assertLessEqual(distance, fuzzyMatch)


class JustifiedLogFormatterTests(unittest.TestCase):
    """Unittests for :class:`bridgedb.util.JustifiedLogFormatter`."""

    def setUp(self):
        # name, level, path, lineno, message, args, exc_info
        self.record = logging.LogRecord('name', logging.INFO, '/foo/bar/baz',
                                        12345, 'This is a message', None, None)

    def test_util_JustifiedLogFormatter(self):
        formatter = util.JustifiedLogFormatter()
        self.assertIsInstance(formatter, logging.Formatter)

    def test_util_JustifiedLogFormatter_logThreads(self):
        formatter = util.JustifiedLogFormatter(logThreads=True)
        self.assertIsInstance(formatter, logging.Formatter)

    def test_util_JustifiedLogFormatter_formatCallingFuncName(self):
        formatter = util.JustifiedLogFormatter()
        record = formatter._formatCallingFuncName(self.record)
        self.assertIsInstance(formatter, logging.Formatter)
        self.assertIsInstance(record, logging.LogRecord)

    def test_util_JustifiedLogFormatter_format(self):
        formatter = util.JustifiedLogFormatter()
        formatted = formatter.format(self.record)
        self.assertIsInstance(formatter, logging.Formatter)
        self.assertIsInstance(formatted, basestring)
        self.assertNotEqual(formatted, '')
        self.assertTrue('INFO' in formatted)
        self.assertTrue('This is a message' in formatted)



# -*- coding: utf-8 -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
#           please also see AUTHORS file
# :copyright: (c) 2013, Isis Lovecruft
#             (c) 2007-2013, The Tor Project, Inc.
#             (c) 2007-2013, all entities within the AUTHORS file
# :license: 3-Clause BSD, see LICENSE for licensing information

"""Unittests for the :mod:`bridgedb.email.dkim` module."""

import io

from twisted.mail.smtp import rfc822
from twisted.trial import unittest

from bridgedb.email import dkim


class CheckDKIMTests(unittest.TestCase):
    """Tests for :func:`email.server.checkDKIM`."""

    def setUp(self):
        """Create fake email, distributor, and associated context data."""
        self.goodMessage = """\
From: user@gmail.com
To: bridges@localhost
X-DKIM-Authentication-Results: pass
Subject: testing

get bridges
"""
        self.badMessage = """\
From: user@gmail.com
To: bridges@localhost
Subject: testing

get bridges
"""
        self.domainRules = {
            'gmail.com': ["ignore_dots", "dkim"],
            'example.com': [],
            'localhost': [],
        }

    def _createMessage(self, messageString):
        """Create an ``rfc822.Message`` from a string."""
        messageIO = io.StringIO(unicode(messageString))
        return rfc822.Message(messageIO)

    def test_checkDKIM_good(self):
        message = self._createMessage(self.goodMessage)
        result = dkim.checkDKIM(message,
                                self.domainRules.get("gmail.com"))
        self.assertTrue(result)
                                         

    def test_checkDKIM_bad(self):
        message = self._createMessage(self.badMessage)
        result = dkim.checkDKIM(message,
                                self.domainRules.get("gmail.com"))
        self.assertIs(result, False)

    def test_checkDKIM_dunno(self):
        """A ``X-DKIM-Authentication-Results: dunno`` header should return
        False.
        """
        messageList = self.badMessage.split('\n')
        messageList[2] = "X-DKIM-Authentication-Results: dunno"
        message = self._createMessage('\n'.join(messageList))
        result = dkim.checkDKIM(message,
                                self.domainRules.get("gmail.com"))
        self.assertIs(result, False)

    def test_checkDKIM_good_dunno(self):
        """A good DKIM verification header, *plus* an
        ``X-DKIM-Authentication-Results: dunno`` header should return False.
        """
        messageList = self.badMessage.split('\n')
        messageList.insert(2, "X-DKIM-Authentication-Results: dunno")
        message = self._createMessage('\n'.join(messageList))
        result = dkim.checkDKIM(message,
                                self.domainRules.get("gmail.com"))
        self.assertIs(result, False)



# -*- coding: utf-8 -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
#           please also see AUTHORS file
# :copyright: (c) 2013, Isis Lovecruft
#             (c) 2007-2013, The Tor Project, Inc.
#             (c) 2007-2013, all entities within the AUTHORS file
# :license: 3-Clause BSD, see LICENSE for licensing information

"""Unittests for the :mod:`bridgedb.email.server` module."""

from __future__ import print_function

import socket
import string
import types

from twisted.python import log
from twisted.internet import defer
from twisted.internet import reactor
from twisted.mail.smtp import IMessage
from twisted.mail.smtp import SMTPBadRcpt
from twisted.mail.smtp import SMTPBadSender
from twisted.mail.smtp import User
from twisted.mail.smtp import Address
from twisted.mail.smtp import rfc822
from twisted.test import proto_helpers
from twisted.trial import unittest

from zope.interface import implementedBy

from bridgedb.Dist import EmailBasedDistributor
from bridgedb.Dist import TooSoonEmail
from bridgedb.email import server
from bridgedb.parse.addr import BadEmail
from bridgedb.schedule import Unscheduled
from bridgedb.test.util import fileCheckDecorator
from bridgedb.test.util import TestCaseMixin
from bridgedb.test.email_helpers import _createConfig
from bridgedb.test.email_helpers import _createMailServerContext


class SMTPMessageTests(unittest.TestCase):
    """Unittests for :class:`bridgedb.email.server.SMTPMessage`."""

    def setUp(self):
        self.config = _createConfig()
        self.context = _createMailServerContext(self.config)
        self.message = server.SMTPMessage(self.context,
                                          canonicalFromSMTP='localhost')
        self.line = string.ascii_lowercase

    def tearDown(self):
        """Re-enable safelogging in between test runs."""
        server.safelog.setSafeLogging(True)

    def test_SMTPMessage_init(self):
        """Our ``message`` attribute should be a ``SMTPMessage`` object, and
        ``message.responder`` should be a
        :class:`bridgedb.email.autoresponder.SMTPAutoresponder`.
        """
        self.assertIsInstance(self.message, server.SMTPMessage)
        self.assertIsInstance(self.message.responder,
                              server.autoresponder.SMTPAutoresponder)

    def test_SMTPMessage_IMessage_interface(self):
        """``SMTPMessage`` should implement ``twisted.mail.smtp.IMessage``."""
        self.assertTrue(IMessage.implementedBy(server.SMTPMessage))

    def test_SMTPMessage_lineReceived_withSafelog(self):
        """Test sending a line of text to ``SMTPMessage.lineReceived`` with
        safelogging enabled.
        """
        server.safelog.setSafeLogging(True)
        self.message.lineReceived(self.line)
        self.assertEqual(self.message.nBytes, 26)
        self.assertTrue(self.line in self.message.lines)

    def test_SMTPMessage_lineReceived_withoutSafelog(self):
        """Test sending a line of text to ``SMTPMessage.lineReceived`` with
        safelogging disnabled.
        """
        server.safelog.setSafeLogging(False)
        for _ in range(3):
            self.message.lineReceived(self.line)
        self.assertEqual(self.message.nBytes, 3*26)
        self.assertTrue(self.line in self.message.lines)

    def test_SMTPMessage_eomReceived(self):
        """Calling ``oemReceived`` should return a deferred."""
        self.message.lineReceived(self.line)
        self.assertIsInstance(self.message.eomReceived(),
                              defer.Deferred)

    def test_SMTPMessage_getIncomingMessage(self):
        """``getIncomingMessage`` should return a ``rfc822.Message``."""
        self.message.lineReceived(self.line)
        self.assertIsInstance(self.message.getIncomingMessage(),
                              rfc822.Message)


class SMTPIncomingDeliveryTests(unittest.TestCase):
    """Unittests for :class:`email.server.SMTPIncomingDelivery`."""

    def setUp(self):
        """Set up our :class:`server.SMTPIncomingDelivery` instance, and reset the
        following ``TestCase`` attributes to ``None``:
            - ``helo``
            - ``proto``
            - ``origin``
            - ``user``
        """
        self.config = _createConfig()
        self.context = _createMailServerContext(self.config)
        self.delivery = server.SMTPIncomingDelivery()

        self.helo = None
        self.proto = None
        self.origin = None
        self.user = None

    def tearDown(self):
        """Reset all TestCase instance attributes between each test run."""
        self.helo = None
        self.proto = None
        self.origin = None
        self.user = None

    def _createProtocolWithHost(self, host):
        """Mock a Protocol which has a ``host`` attribute.

        We don't currently use any of the ``IProtocol`` methods of the
        returned ``twisted.test.proto_helpers.AccumulatingProtocol``, and so
        this could be any class, although a mocked ``IProtocol`` implementer
        was chosen for completeness and realism's sakes.

        :param str host: A domain name or IP address.
        :rtype: :api:`twisted.test.proto_helpers.AccumulatingProtocol`
        :returns: A Protocol instance which has its ``host`` attribute set to
            the given **host**, so that an :api:`twisted.mail.smtp.User` can
            be constructed with it.
        """
        self.proto = proto_helpers.AccumulatingProtocol()
        self.proto.host = host

    def _createUser(self, username, domain, ipaddress):
        """Create a ``twisted.mail.smtp.User`` for testing.

        :param str username: The local part of the client's email address.
        :param str domain: The host part of the client's email address.
        :param str ipaddress: The IP address of the client's mail server.
        """
        self.helo = (domain, ipaddress)
        self._createProtocolWithHost(domain)
        self.origin = Address('@'.join((username, domain,)))
        self.user = User(username, self.helo, self.proto, self.origin)

    def _setUpMAILFROM(self):
        """Set up the parameters for emulating a connected client sending a
        SMTP 'MAIL FROM:' command to us.

        The default is to emulate sending: ``MAIL FROM: client@example.com``.
        """
        self.helo = ('localhost', '127.0.0.1')
        self.origin = server.smtp.Address('client@example.com')
        self.delivery.setContext(self.context)

    def _setUpRCPTTO(self, username=None, domain=None, ip=None):
        """Set up the parameters for emulating a connected client sending a
        SMTP 'RCPT TO:' command to us.

        The default is to emulate sending: ``RCPT TO: bridges@localhost``.
        """
        name = username if username is not None else self.config.EMAIL_USERNAME
        host = domain if domain is not None else 'localhost'
        addr = ip if ip is not None else '127.0.0.1'
        self._createUser(name, host, ip)
        self.delivery.setContext(self.context)

    def test_SMTPIncomingDelivery_init(self):
        """After calling :meth:`server.SMTPIncomingDelivery.__init__`, we should have a
        :class:`server.SMTPIncomingDelivery` object instance.
        """
        self.assertIsInstance(self.delivery, server.SMTPIncomingDelivery)

    def test_SMTPIncomingDelivery_setContext(self):
        """Calling :meth:`server.SMTPIncomingDelivery.setContext` should set
        the :ivar:`SMTPIncomingDelivery.context` attribute.

        The ``SMTPIncomingDelivery.context`` should be a :class:`server.MailServerContext`,
        and it should have relevant settings from the config file stored
        within it.
        """
        self.delivery.setContext(self.context)
        self.assertIsInstance(self.delivery.context, server.MailServerContext)
        self.assertEqual(self.delivery.context.smtpFromAddr,
                         self.config.EMAIL_SMTP_FROM_ADDR)

    def test_SMTPIncomingDelivery_receivedHeader(self):
        """The email resulting from a SMTPIncomingDelivery, the latter received from
        ``'client@example.com'`` should contain a header stating:
        ``'Received: from example.com'``.
        """
        self._createUser('client', 'example.com', '127.0.0.1')
        hdr = self.delivery.receivedHeader(self.helo, self.origin, [self.user,])
        self.assertSubstring("Received: from example.com", hdr)

    def test_SMTPIncomingDelivery_validateFrom(self):
        """A valid origin should be stored as ``SMTPIncomingDelivery.fromCanonical``."""
        self._setUpMAILFROM()
        self.delivery.validateFrom(self.helo, self.origin)
        self.assertEqual(self.delivery.fromCanonicalSMTP, 'example.com')

    def test_SMTPIncomingDelivery_validateFrom_unsupportedDomain(self):
        """A domain not in our canon should raise a SMTPBadSender."""
        self._setUpMAILFROM()
        origin = server.smtp.Address('throwing.pickles@yo.mama')
        self.assertRaises(SMTPBadSender,
                          self.delivery.validateFrom, self.helo, origin)

    def test_SMTPIncomingDelivery_validateFrom_origin_notAdressType(self):
        """A non ``twisted.mail.smtp.Address`` origin should raise an
        AttributeError exception.
        """
        self._setUpMAILFROM()
        origin = 'throwing.pickles@yo.mama'
        self.delivery.validateFrom(self.helo, origin)

    def test_SMTPIncomingDelivery_validateTo(self):
        """Should return a callable that results in a SMTPMessage."""
        self._setUpRCPTTO()
        validated = self.delivery.validateTo(self.user)
        self.assertIsInstance(validated, types.FunctionType)
        self.assertIsInstance(validated(), server.SMTPMessage)

    def test_SMTPIncomingDelivery_validateTo_plusAddress(self):
        """Should return a callable that results in a SMTPMessage."""
        self._setUpRCPTTO('bridges+ar')
        validated = self.delivery.validateTo(self.user)
        self.assertIsInstance(validated, types.FunctionType)
        self.assertIsInstance(validated(), server.SMTPMessage)

    def test_SMTPIncomingDelivery_validateTo_badUsername_plusAddress(self):
        """'givemebridges+zh_cn@...' should raise an SMTPBadRcpt exception."""
        self._setUpRCPTTO('givemebridges+zh_cn')
        self.assertRaises(SMTPBadRcpt, self.delivery.validateTo, self.user)

    def test_SMTPIncomingDelivery_validateTo_badUsername(self):
        """A :class:`server.SMTPIncomingDelivery` which sends a SMTP
        ``RCPT TO: yo.mama@localhost`` should raise a
        ``twisted.mail.smtp.SMTPBadRcpt`` exception.
        """
        self._setUpRCPTTO('yo.mama')
        self.assertRaises(SMTPBadRcpt, self.delivery.validateTo, self.user)

    def test_SMTPIncomingDelivery_validateTo_notOurDomain(self):
        """An SMTP ``RCPT TO: bridges@forealsi.es`` should raise an SMTPBadRcpt
        exception.
        """
        self._setUpRCPTTO('bridges', 'forealsi.es')
        self.assertRaises(SMTPBadRcpt, self.delivery.validateTo, self.user)

    def test_SMTPIncomingDelivery_validateTo_subdomain(self):
        """An SMTP ``RCPT TO: bridges@subdomain.localhost`` should be allowed.
        """
        self._setUpRCPTTO('bridges', 'subdomain.localhost')
        validated = self.delivery.validateTo(self.user)
        self.assertIsInstance(validated, types.FunctionType)
        self.assertIsInstance(validated(), server.SMTPMessage)


class SMTPTestCaseMixin(TestCaseMixin):
    """Utility methods for use within any subclasses of
    :api:`twisted.trial.unittest.TestCase` which test SMTP.

    To use me, subclass :api:`twisted.trial.unittest.TestCase` and mix me into
    the middle of your class inheritance declarations, like so::

        class ExampleSMTPTests(SMTPTestCaseMixin, unittest.TestCase):
            pass

    and then make certain that your ``TestCase`` subclass has its ``proto``
    and attribute assigned properly::

        class ExampleSMTPTests(SMTPTestCaseMixin, unittest.TestCase):
            def setUp(self):
                factory = twisted.mail.smtp.SMTPIncomingServerFactory()
                self.proto = self.factory.buildProtocol(('127.0.0.1', 0))


    :ivar proto: A :api:`Protocol <twisted.internet.protocol.Protocol>`
        associated with a
        :api:`ServerFactory <twisted.internet.protocol.ServerFactory>`.
    :ivar transport: Anything that implements
        :api:`ITransport <twisted.internet.interfaces.ITransport>`. The default
        is a :api:`twisted.test.proto_helpers.StringTransportWithDisconection`.
    :ivar str smtpFromAddr: The default email address for the server.
    """

    proto = None
    transport = proto_helpers.StringTransportWithDisconnection()
    smtpFromAddr = None

    def tearDown(self):
        """Cleanup method after each ``test_*`` method runs; removes timed out
        connections on the reactor and clears the :ivar:`transport`.
        """
        self.transport.clear()  # Clear bytes from the transport.

        for delay in reactor.getDelayedCalls():
            try:
                delay.cancel()
            except (AlreadyCalled, AlreadyCancelled):
                pass

    def _buildEmail(self, fromAddr=None, toAddr=None, subject=None, body=None):
        """Creates email text (including headers) for use in an SMTP DATA
         segment. Includes the SMTP DATA EOM command ('.') at the end. If no
         keyword arguments are given, the defaults are fairly sane.

        Suitable for testing a :class:`bridgedb.email.server.SMTPIncomingServerFactory`.

        :param str fromAddr: An email address for the 'From:' header.
        :param str toAddr: An email address for the 'To:' header.
        :param str subject: The contents of the 'Subject:' header.
        :param str body: The contents of the email body.
        :rtype: str
        :returns: The email text.
        """
        fromAddr = fromAddr if fromAddr else 'testing@localhost'
        toAddr = toAddr if toAddr else self.smtpFromAddr
        subject = subject if subject else 'testing testing one two three'
        body = body if body else 'get bridges'

        contents = ['From: %s' % fromAddr,
                    'To: %s' % toAddr,
                    'Subject: %s' % subject,
                    '\r\n %s' % body,
                    '.']  # SMTP DATA EOM command
        emailText = self.proto.delimiter.join(contents)
        return emailText

    def _buildSMTP(self, commands):
        """Format a list of SMTP protocol commands into a string, using the proper
         protocol delimiter.

        :param list commands: A list of raw SMTP-protocol command lines.
        :rtype: str
        :returns: The string for sending those **commands**, suitable for
            giving to a :api:`twisted.internet.Protocol.dataReceived` method.
        """
        data = self.proto.delimiter.join(commands) + self.proto.delimiter
        return data

    def _test(self, commands, expected, noisy=False):
        """Send the SMTP **commands** to the ``dataReceived`` method of your
         TestCase's protocol (this must be the `proto` attribute of your
         `TestCase`, i.e. this uses ``TestCase.proto.dataReceived``). Next,
         check that the substring which is **expected** to be within the
         server's output matches what was received from :ivar`transport`.

        :param list commands: A sequence of raw SMTP command lines. This will
            automatically be passed to :meth:`_buildSMTP`.
        :param str expected: A substring which should occur in the "server"
            output (taken from the :ivar:`transport`).
        :param bool noisy: If ``True``, print the conversation between the
            "client" and the "server" in a nicely formatted manner.
        """
        data = self._buildSMTP(commands)
        self.proto.dataReceived(data)
        recv = self.transport.value()

        if noisy:
            client = data.replace('\r\n', '\r\n ')
            server = recv.replace('\r\n', '\r\n\t\t ')
            print('\n CLIENT --------->', '\n %s' % client)
            print('\t\t', '<--------- SERVER', '\n\t\t %s' % server)

        self.assertSubstring(expected, recv)


class SMTPIncomingServerFactoryTests(SMTPTestCaseMixin, unittest.TestCase):
    """Unittests for :class:`bridgedb.email.server.SMTPIncomingServerFactory`."""

    def setUp(self):
        """Set up a localhost SMTPIncomingServerFactory handler incoming SMTP
        connections.
        """
        config = _createConfig()
        context = _createMailServerContext(config)
        factory = server.SMTPIncomingServerFactory()
        factory.setContext(context)
        factory.protocol.timeout = None  # Otherwise the reactor gets dirty

        self.smtpFromAddr = context.smtpFromAddr  # 'bridges@localhost'
        self.proto = factory.buildProtocol(('127.0.0.1', 0))
        self.transport = proto_helpers.StringTransportWithDisconnection()
        self.proto.setTimeout(None)
        # Set the protocol; StringTransportWithDisconnection is a bit janky:
        self.transport.protocol = self.proto
        self.proto.makeConnection(self.transport)

    def test_SMTPIncomingServerFactory_HELO_localhost(self):
        """Send 'HELO localhost' to the server's transport."""
        ip = self.transport.getPeer().host
        self._test(['HELO localhost'],
                   "Hello %s, nice to meet you" % ip)

    def test_SMTPIncomingServerFactory_MAIL_FROM_testing_at_localhost(self):
        """Send 'MAIL FROM: human@localhost'."""
        self._test(['HELO localhost',
                    'MAIL FROM: testing@localhost'],
                   "250 Sender address accepted")

    def test_SMTPIncomingServerFactory_MAIL_FROM_testing_at_gethostname(self):
        """Send 'MAIL FROM: human@hostname' for the local hostname."""
        hostname = socket.gethostname() or "computer"
        self._test(['HELO localhost',
                    'MAIL FROM: testing@%s' % hostname],
                   "250 Sender address accepted")

    def test_SMTPIncomingServerFactory_MAIL_FROM_testing_at_ipaddress(self):
        """Send 'MAIL FROM: human@ipaddr' for the loopback IP address."""
        hostname = 'localhost'
        self._test(['HELO localhost',
                    'MAIL FROM: testing@%s' % hostname],
                   "250 Sender address accepted")

    def test_SMTPIncomingServerFactory_RCPT_TO_context_smtpFromAddr(self):
        """Send 'RCPT TO:' with the context.smtpFromAddr."""
        self._test(['HELO localhost',
                    'MAIL FROM: testing@localhost',
                    'RCPT TO: %s' % self.smtpFromAddr],
                   "250 Recipient address accepted")

    def test_SMTPIncomingServerFactory_DATA_blank(self):
        """A DATA command with nothing after it should receive::
            '354 Continue'
        in response.
        """
        self._test(['HELO localhost',
                    'MAIL FROM: testing@localhost',
                    'RCPT TO: %s' % self.smtpFromAddr,
                    "DATA"],
                   "354 Continue")

    def test_SMTPIncomingServerFactory_DATA_get_help(self):
        """A DATA command with ``'get help'`` in the email body should
        receive::
            '250 Delivery in progress'
        in response.
        """
        emailText = self._buildEmail(body="get help")
        self._test(['HELO localhost',
                    'MAIL FROM: testing@localhost',
                    'RCPT TO: %s' % self.smtpFromAddr,
                    "DATA", emailText],
                   "250 Delivery in progress",
                   noisy=True)

    def test_SMTPIncomingServerFactory_DATA_get_transport_obfs3(self):
        """A DATA command with ``'get transport obfs3'`` in the email body
        should receive::
            '250 Delivery in progress'
        in response.
        """
        emailText = self._buildEmail(body="get transport obfs3")
        self._test(['HELO localhost',
                    'MAIL FROM: testing@localhost',
                    'RCPT TO: %s' % self.smtpFromAddr,
                    "DATA", emailText],
                   "250 Delivery in progress",
                   noisy=True)

    def test_SMTPIncomingServerFactory_DATA_To_bridges_plus_zh_CN(self):
        """Test sending to 'bridges+zh_CN' address for Chinese translations."""
        # TODO: Add tests which use '+' syntax in mailTo in order to test
        # email translations. Do this when some strings have been translated.
        emailTo = list(self.smtpFromAddr.partition('@'))
        emailTo.insert(1, '+zh_CN')
        emailTo = ''.join(emailTo)
        emailText = self._buildEmail(toAddr=emailTo)
        self._test(['HELO localhost',
                    'MAIL FROM: testing@localhost',
                    'RCPT TO: %s' % emailTo,
                    "DATA", emailText],
                   "250 Delivery in progress",
                   noisy=True)

    def test_SMTPIncomingServerFactory_DATA_get_bridges_QUIT(self):
        """Test sending 'DATA' with 'get bridges', then sending 'QUIT'."""
        emailText = self._buildEmail()
        self._test(['HELO localhost',
                    'MAIL FROM: testing@localhost',
                    'RCPT TO: %s' % self.smtpFromAddr,
                    "DATA", emailText,
                    "QUIT"],
                   "221 See you later",
                   noisy=True)


class EmailServerServiceTests(SMTPTestCaseMixin, unittest.TestCase):
    """Unittests for :func:`bridgedb.email.server.addServer`."""

    def setUp(self):
        """Create a server.MailServerContext and EmailBasedDistributor."""
        self.config = _createConfig()
        self.context = _createMailServerContext(self.config)
        self.smtpFromAddr = self.context.smtpFromAddr  # 'bridges@localhost'
        self.sched = Unscheduled()
        self.dist = self.context.distributor

    def tearDown(self):
        """Kill all connections with fire."""
        if self.transport:
            self.transport.loseConnection()
        super(EmailServerServiceTests, self).tearDown()
        # FIXME: this is definitely not how we're supposed to do this, but it
        # kills the DirtyReactorAggregateErrors.
        reactor.disconnectAll()
        reactor.runUntilCurrent()

    def test_addServer(self):
        """Call :func:`bridgedb.email.server.addServer` to test startup."""
        factory = server.addServer(self.config, self.dist, self.sched)
        factory.timeout = None
        factory.protocol.timeout = None  # Or else the reactor gets dirty

        self.proto = factory.buildProtocol(('127.0.0.1', 0))
        self.proto.setTimeout(None)
        # Set the transport's protocol, because
        # StringTransportWithDisconnection is a bit janky:
        self.transport.protocol = self.proto
        self.proto.makeConnection(self.transport)

        serverHost = socket.gethostname()
        self._test(['HELO %s' % serverHost,
                    'MAIL FROM: testing@%s' % serverHost,
                    'RCPT TO: %s' % self.smtpFromAddr,
                    "DATA", self._buildEmail(body="get transport obfs3")],
                   "250 Delivery in progress",
                   noisy=True)



# -*- coding: utf-8 -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
#           please also see AUTHORS file
# :copyright: (c) 2013, Isis Lovecruft
#             (c) 2007-2013, The Tor Project, Inc.
#             (c) 2007-2013, all entities within the AUTHORS file
# :license: 3-Clause BSD, see LICENSE for licensing information

"""Unittests for the :mod:`bridgedb.email.autoresponder` module."""

from __future__ import print_function

import io
import os
import shutil

from twisted.internet import defer
from twisted.mail.smtp import Address
from twisted.python.failure import Failure
from twisted.trial import unittest
from twisted.test import proto_helpers

from bridgedb.email import autoresponder
from bridgedb.email.server import SMTPMessage
from bridgedb.Dist import TooSoonEmail
from bridgedb.test.email_helpers import _createConfig
from bridgedb.test.email_helpers import _createMailServerContext
from bridgedb.test.email_helpers import DummyEmailDistributorWithState


class CreateResponseBodyTests(unittest.TestCase):
    """Tests for :func:`bridgedb.email.autoresponder.createResponseBody`."""

    def _moveGPGTestKeyfile(self):
        here          = os.getcwd()
        topDir        = here.rstrip('_trial_temp')
        self.gpgFile  = os.path.join(topDir, '.gnupg', 'TESTING.subkeys.sec')
        self.gpgMoved = os.path.join(here, 'TESTING.subkeys.sec')
        shutil.copy(self.gpgFile, self.gpgMoved)

    def setUp(self):
        """Create fake email, distributor, and associated context data."""
        self._moveGPGTestKeyfile()
        self.toAddress = "user@example.com"
        self.config = _createConfig()
        self.ctx = _createMailServerContext(self.config)
        self.distributor = self.ctx.distributor

    def _getIncomingLines(self, clientAddress="user@example.com"):
        """Generate the lines of an incoming email from **clientAddress**."""
        self.toAddress = Address(clientAddress)
        lines = [
            "From: %s" % clientAddress,
            "To: bridges@localhost",
            "Subject: testing",
            "",
            "get bridges",
        ]
        return lines

    def test_createResponseBody_getKey(self):
        """A request for 'get key' should receive our GPG key."""
        lines = self._getIncomingLines()
        lines[4] = "get key"
        ret = autoresponder.createResponseBody(lines, self.ctx, self.toAddress)
        self.assertSubstring('-----BEGIN PGP PUBLIC KEY BLOCK-----', ret)

    def test_createResponseBody_bridges_invalid(self):
        """An invalid request for 'transport obfs3' should get help text."""
        lines = self._getIncomingLines("testing@localhost")
        lines[4] = "transport obfs3"
        ret = autoresponder.createResponseBody(lines, self.ctx, self.toAddress)
        self.assertSubstring("COMMANDs", ret)

    def test_createResponseBody_bridges_obfs3(self):
        """A request for 'get transport obfs3' should receive a response."""
        lines = self._getIncomingLines("testing@localhost")
        lines[4] = "get transport obfs3"
        ret = autoresponder.createResponseBody(lines, self.ctx, self.toAddress)
        self.assertSubstring("Here are your bridges", ret)
        self.assertSubstring("obfs3", ret)

    def test_createResponseBody_bridges_obfsobfswebz(self):
        """We should only pay attention to the *last* in a crazy request."""
        lines = self._getIncomingLines("testing@localhost")
        lines[4] = "get unblocked webz"
        lines.append("get transport obfs2")
        lines.append("get transport obfs3")
        ret = autoresponder.createResponseBody(lines, self.ctx, self.toAddress)
        self.assertSubstring("Here are your bridges", ret)
        self.assertSubstring("obfs3", ret)

    def test_createResponseBody_bridges_obfsobfswebzipv6(self):
        """We should *still* only pay attention to the *last* request."""
        lines = self._getIncomingLines("testing@localhost")
        lines[4] = "transport obfs3"
        lines.append("get unblocked webz")
        lines.append("get ipv6")
        lines.append("get transport obfs2")
        ret = autoresponder.createResponseBody(lines, self.ctx, self.toAddress)
        self.assertSubstring("Here are your bridges", ret)
        self.assertSubstring("obfs2", ret)

    def test_createResponseBody_two_requests_TooSoonEmail(self):
        """The same client making two requests in a row should receive a
        rate-limit warning for the second response.
        """
        # Set up a mock distributor which keeps state:
        dist = DummyEmailDistributorWithState()
        ctx = _createMailServerContext(self.config, dist)

        lines = self._getIncomingLines("testing@localhost")
        first = autoresponder.createResponseBody(lines, ctx, self.toAddress)
        self.assertSubstring("Here are your bridges", first)
        second = autoresponder.createResponseBody(lines, ctx, self.toAddress)
        self.assertSubstring("Please slow down", second)

    def test_createResponseBody_three_requests_TooSoonEmail(self):
        """Alice making a request, next Bob making a request, and then Alice again,
        should result in both of their first requests getting them bridges,
        and then Alice's second request gets her a rate-limit warning email.
        """
        # Set up a mock distributor which keeps state:
        dist = DummyEmailDistributorWithState()
        ctx = _createMailServerContext(self.config, dist)

        aliceLines = self._getIncomingLines("alice@localhost")
        aliceFirst = autoresponder.createResponseBody(aliceLines, ctx,
                                                      self.toAddress)
        self.assertSubstring("Here are your bridges", aliceFirst)

        bobLines = self._getIncomingLines("bob@localhost")
        bobFirst = autoresponder.createResponseBody(bobLines, ctx,
                                                    self.toAddress)
        self.assertSubstring("Here are your bridges", bobFirst)

        aliceSecond = autoresponder.createResponseBody(aliceLines, ctx,
                                                       self.toAddress)
        self.assertSubstring("Please slow down", aliceSecond)

    def test_createResponseBody_three_requests_IgnoreEmail(self):
        """The same client making three requests in a row should receive a
        rate-limit warning for the second response, and then nothing for every
        request thereafter.
        """
        # Set up a mock distributor which keeps state:
        dist = DummyEmailDistributorWithState()
        ctx = _createMailServerContext(self.config, dist)

        lines = self._getIncomingLines("testing@localhost")
        first = autoresponder.createResponseBody(lines, ctx, self.toAddress)
        self.assertSubstring("Here are your bridges", first)
        second = autoresponder.createResponseBody(lines, ctx, self.toAddress)
        self.assertSubstring("Please slow down", second)
        third = autoresponder.createResponseBody(lines, ctx, self.toAddress)
        self.assertIsNone(third)
        fourth = autoresponder.createResponseBody(lines, ctx, self.toAddress)
        self.assertIsNone(fourth)


class EmailResponseTests(unittest.TestCase):
    """Tests for ``generateResponse()`` and ``EmailResponse``."""

    def setUp(self):
        self.fromAddr = "bridges@torproject.org"
        self.clientAddr = "user@example.com"
        self.body = """\
People think that time is strictly linear, but, in reality, it's actually just
a ball of timey-wimey, wibbly-warbly... stuff."""

    def tearDown(self):
        autoresponder.safelog.safe_logging = True

    def test_EmailResponse_generateResponse(self):
        response = autoresponder.generateResponse(self.fromAddr,
                                                  self.clientAddr,
                                                  self.body)
        self.assertIsInstance(response, autoresponder.EmailResponse)

    def test_EmailResponse_generateResponse_noSafelog(self):
        autoresponder.safelog.safe_logging = False
        response = autoresponder.generateResponse(self.fromAddr,
                                                  self.clientAddr,
                                                  self.body)
        self.assertIsInstance(response, autoresponder.EmailResponse)

    def test_EmailResponse_generateResponse_mailfile(self):
        response = autoresponder.generateResponse(self.fromAddr,
                                                  self.clientAddr,
                                                  self.body)
        self.assertIsInstance(response.mailfile, (io.BytesIO, io.StringIO))

    def test_EmailResponse_generateResponse_withInReplyTo(self):
        response = autoresponder.generateResponse(self.fromAddr,
                                                  self.clientAddr,
                                                  self.body,
                                                  messageID="NSA")
        contents = str(response.readContents()).replace('\x00', '')
        self.assertIsInstance(response.mailfile, (io.BytesIO, io.StringIO))
        self.assertSubstring("In-Reply-To: NSA", contents)

    def test_EmailResponse_generateResponse_readContents(self):
        response = autoresponder.generateResponse(self.fromAddr,
                                                  self.clientAddr,
                                                  self.body)
        contents = str(response.readContents()).replace('\x00', '')
        self.assertSubstring('timey-wimey, wibbly-warbly... stuff.', contents)

    def test_EmailResponse_additionalHeaders(self):
        response = autoresponder.EmailResponse()
        response.writeHeaders(self.fromAddr, self.clientAddr,
                              subject="Re: echelon", inReplyTo="NSA",
                              X_been_there="They were so 2004")
        contents = str(response.readContents()).replace('\x00', '')
        self.assertIsInstance(response.mailfile, (io.BytesIO, io.StringIO))
        self.assertSubstring("In-Reply-To: NSA", contents)
        self.assertSubstring("X-been-there: They were so 2004", contents)

    def test_EmailResponse_close(self):
        """Calling EmailResponse.close() should close the ``mailfile`` and set
        ``closed=True``.
        """
        response = autoresponder.EmailResponse()
        self.assertEqual(response.closed, False)
        response.close()
        self.assertEqual(response.closed, True)
        self.assertRaises(ValueError, response.write, self.body)

    def test_EmailResponse_read(self):
        """Calling EmailResponse.read() should read bytes from the file."""
        response = autoresponder.EmailResponse()
        response.write(self.body)
        response.rewind()
        contents = str(response.read()).replace('\x00', '')
        # The newlines in the email body should have been replaced with
        # ``EmailResponse.delimiter``.
        delimited = self.body.replace('\n', response.delimiter) \
                    + response.delimiter
        self.assertEqual(delimited, contents)

    def test_EmailResponse_read_three_bytes(self):
        """EmailResponse.read(3) should read three bytes from the file."""
        response = autoresponder.EmailResponse()
        response.write(self.body)
        response.rewind()
        contents = str(response.read(3)).replace('\x00', '')
        self.assertEqual(contents, self.body[:3])

    def test_EmailResponse_write(self):
        """Calling EmailResponse.write() should write to the mailfile."""
        response = autoresponder.EmailResponse()
        response.write(self.body)
        contents = str(response.readContents()).replace('\x00', '')
        # The newlines in the email body should have been replaced with
        # ``EmailResponse.delimiter``.
        delimited = self.body.replace('\n', response.delimiter) \
                    + response.delimiter
        self.assertEqual(delimited, contents)

    def test_EmailResponse_write_withRetNewlines(self):
        """Calling EmailResponse.write() with '\r\n' in the lines should call
        writelines(), which splits up the lines and then calls write() again.
        """
        response = autoresponder.EmailResponse()
        response.write(self.body.replace('\n', '\r\n'))
        contents = str(response.readContents()).replace('\x00', '')
        # The newlines in the email body should have been replaced with
        # ``EmailResponse.delimiter``.
        delimited = self.body.replace('\n', response.delimiter) \
                    + response.delimiter
        self.assertEqual(delimited, contents)

    def test_EmailResponse_writelines_list(self):
        """Calling EmailResponse.writelines() with a list should write the
        concatenated contents of the list into the mailfile.
        """
        response = autoresponder.EmailResponse()
        response.writelines(self.body.split('\n'))
        contents = str(response.readContents()).replace('\x00', '')
        # The newlines in the email body should have been replaced with
        # ``EmailResponse.delimiter``.
        delimited = self.body.replace('\n', response.delimiter) \
                    + response.delimiter
        self.assertEqual(delimited, contents)


class SMTPAutoresponderTests(unittest.TestCase):
    """Unittests for :class:`bridgedb.email.autoresponder.SMTPAutoresponder`."""

    timeout = 10

    def setUp(self):
        self.config = _createConfig()
        self.context = _createMailServerContext(self.config)
        self.message = SMTPMessage(self.context)

    def _getIncomingLines(self, clientAddress="user@example.com"):
        """Generate the lines of an incoming email from **clientAddress**."""
        lines = [
            "From: %s" % clientAddress,
            "To: bridges@localhost",
            "Subject: testing",
            "",
            "get bridges",
        ]
        self.message.lines = lines

    def _setUpResponder(self):
        """Set up the incoming message of our autoresponder.

        This is necessary because normally our SMTP server acts as a line
        protocol, waiting for an EOM which sets off a chain of deferreds
        resulting in the autoresponder sending out the response. This should
        be called after :meth:`_getIncomingLines` so that we can hook into the
        SMTP protocol without actually triggering all the deferreds.
        """
        self.message.message = self.message.getIncomingMessage()
        self.responder = self.message.responder
        # The following are needed to provide client disconnection methods for
        # the call to ``twisted.mail.smtp.SMTPClient.sendError`` in
        # ``bridgedb.email.autoresponder.SMTPAutoresponder.sendError``:
        #protocol = proto_helpers.AccumulatingProtocol()
        #transport = proto_helpers.StringTransportWithDisconnection()
        self.tr = proto_helpers.StringTransportWithDisconnection()
        # Set the transport's protocol, because
        # StringTransportWithDisconnection is a bit janky:
        self.tr.protocol = self.responder
        self.responder.makeConnection(self.tr)

    def test_SMTPAutoresponder_getMailFrom_notbridgedb_at_yikezors_dot_net(self):
        """SMTPAutoresponder.getMailFrom() for an incoming email sent to any email
        address other than the one we're listening for should return our
        configured address, not the one in the incoming email.
        """
        self._getIncomingLines()
        self.message.lines[1] = 'To: notbridgedb@yikezors.net'
        self._setUpResponder()
        recipient = str(self.responder.getMailFrom())
        self.assertEqual(recipient, self.context.fromAddr)

    def test_SMTPAutoresponder_getMailFrom_givemebridges_at_seriously(self):
        """SMTPAutoresponder.getMailFrom() for an incoming email sent to any email
        address other than the one we're listening for should return our
        configured address, not the one in the incoming email.
        """
        self._getIncomingLines()
        self.message.lines[1] = 'To: givemebridges@serious.ly'
        self._setUpResponder()
        recipient = str(self.responder.getMailFrom())
        self.assertEqual(recipient, self.context.fromAddr)

    def test_SMTPAutoresponder_getMailFrom_bad_address(self):
        """SMTPAutoresponder.getMailFrom() for an incoming email sent to a malformed
        email address should log an smtp.AddressError and then return our
        configured email address.
        """
        self._getIncomingLines()
        self.message.lines[1] = 'To: ><@><<<>>.foo'
        self._setUpResponder()
        recipient = str(self.responder.getMailFrom())
        self.assertEqual(recipient, self.context.fromAddr)

    def test_SMTPAutoresponder_getMailFrom_plus_address(self):
        """SMTPAutoresponder.getMailFrom() for an incoming email sent with a valid
        plus address should respond.
        """
        self._getIncomingLines()
        ours = Address(self.context.fromAddr)
        plus = '@'.join([ours.local + '+zh_cn', ours.domain])
        self.message.lines[1] = 'To: {0}'.format(plus)
        self._setUpResponder()
        recipient = str(self.responder.getMailFrom())
        self.assertEqual(recipient, plus)

    def test_SMTPAutoresponder_getMailFrom_getbridges_at_localhost(self):
        """SMTPAutoresponder.getMailFrom() for an incoming email sent with
        'getbridges+zh_cn@localhost' should be responded to from the default
        address.
        """
        self._getIncomingLines()
        ours = Address(self.context.fromAddr)
        plus = '@'.join(['get' + ours.local + '+zh_cn', ours.domain])
        self.message.lines[1] = 'To: {0}'.format(plus)
        self._setUpResponder()
        recipient = str(self.responder.getMailFrom())
        self.assertEqual(recipient, self.context.fromAddr)

    def test_SMTPAutoresponder_getMailTo_UnsupportedDomain(self):
        """getMailTo() should catch emails from UnsupportedDomains."""
        emailFrom = 'some.dude@un.support.ed'
        self._getIncomingLines(emailFrom)
        self._setUpResponder()
        clients = self.responder.getMailTo()
        self.assertIsInstance(clients, list, (
            "Returned value of SMTPAutoresponder.getMailTo() isn't a list! "
            "Type: %s" % type(clients)))
        self.assertTrue(emailFrom not in clients)
        # The client was from an unsupported domain; they shouldn't be in the
        # clients list:
        self.assertEqual(len(clients), 0,
                         "clients = %s" % repr(clients))

    def test_SMTPAutoresponder_reply_noFrom(self):
        """A received email without a "From:" or "Sender:" header shouldn't
        receive a response.
        """
        self._getIncomingLines()
        self.message.lines[0] = ""
        self._setUpResponder()
        ret = self.responder.reply()
        self.assertIsInstance(ret, defer.Deferred)

    def test_SMTPAutoresponder_reply_badAddress(self):
        """Don't respond to RFC2822 malformed source addresses."""
        self._getIncomingLines("testing*.?\"@example.com")
        self._setUpResponder()
        ret = self.responder.reply()
        # This will call ``self.responder.reply()``:
        #ret = self.responder.incoming.eomReceived()
        self.assertIsInstance(ret, defer.Deferred)

    def test_SMTPAutoresponder_reply_anotherBadAddress(self):
        """Don't respond to RFC2822 malformed source addresses."""
        self._getIncomingLines("Mallory <>>@example.com")
        self._setUpResponder()
        ret = self.responder.reply()
        self.assertIsInstance(ret, defer.Deferred)

    def test_SMTPAutoresponder_reply_invalidDomain(self):
        """Don't respond to RFC2822 malformed source addresses."""
        self._getIncomingLines("testing@exa#mple.com")
        self._setUpResponder()
        ret = self.responder.reply()
        self.assertIsInstance(ret, defer.Deferred)

    def test_SMTPAutoresponder_reply_anotherInvalidDomain(self):
        """Don't respond to RFC2822 malformed source addresses."""
        self._getIncomingLines("testing@exam+ple.com")
        self._setUpResponder()
        ret = self.responder.reply()
        self.assertIsInstance(ret, defer.Deferred)

    def test_SMTPAutoresponder_reply_DKIM_badDKIMheader(self):
        """An email with an 'X-DKIM-Authentication-Result:' header appended
        after the body should not receive a response.
        """
        self._getIncomingLines("testing@gmail.com")
        self.message.lines.append("X-DKIM-Authentication-Result: ")
        self._setUpResponder()
        ret = self.responder.reply()
        self.assertIsInstance(ret, defer.Deferred)

    def test_SMTPAutoresponder_reply_goodDKIMheader(self):
        """An email with a good DKIM header should be responded to."""
        self._getIncomingLines("testing@gmail.com")
        self.message.lines.insert(3, "X-DKIM-Authentication-Result: pass")
        self._setUpResponder()
        ret = self.responder.reply()
        self.assertIsInstance(ret, defer.Deferred)

    def test_SMTPAutoresponder_reply_transport_invalid(self):
        """An invalid request for 'transport obfs3' should get help text."""
        #self.skip = True
        #raise unittest.SkipTest("We need to fake the reactor for this one")

        def cb(success):
            pass
        self._getIncomingLines("testing@example.com")
        self.message.lines[4] = "transport obfs3"
        self._setUpResponder()
        ret = self.responder.reply()
        self.assertIsInstance(ret, defer.Deferred)
        #self.assertSubstring("COMMANDs", ret)
        print(self.tr.value())
        return ret

    def test_SMTPAutoresponder_reply_transport_valid(self):
        """An valid request for 'get transport obfs3' should get obfs3."""
        #self.skip = True
        #raise unittest.SkipTest("We need to fake the reactor for this one")
    
        self._getIncomingLines("testing@example.com")
        self.message.lines[4] = "transport obfs3"
        self._setUpResponder()
        ret = self.responder.reply()
        self.assertIsInstance(ret, defer.Deferred)
        #self.assertSubstring("obfs3", ret)
        print(self.tr.value())
        return ret

    def test_SMTPAutoresponder_sentMail(self):
        """``SMTPAutoresponder.sendMail()`` should handle successes from an
        :api:`twisted.mail.smtp.SMTPSenderFactory`.
        """
        success = (1, [('me@myaddress.com', 250, 'OK',)])
        self._getIncomingLines()
        self._setUpResponder()
        self.responder.sentMail(success)

    def test_SMTPAutoresponder_sendError_fail(self):
        """``SMTPAutoresponder.sendError()`` should handle failures."""
        fail = Failure(ValueError('This failure was sent on purpose.'))
        self._getIncomingLines()
        self._setUpResponder()
        self.responder.sendError(fail)

    def test_SMTPAutoresponder_sendError_exception(self):
        """``SMTPAutoresponder.sendError()`` should handle exceptions."""
        error = ValueError('This error was sent on purpose.')
        self._getIncomingLines()
        self._setUpResponder()
        self.responder.sendError(error)

    def test_SMTPAutoresponder_runChecks_RCPTTO_From_mismatched_domain(self):
        """runChecks() should catch emails where the SMTP 'MAIL FROM:' command
        reported being from an email address at one supported domain and the
        email's 'From:' header reported another domain.
        """
        smtpFrom = 'not.an.evil.bot@yahoo.com'
        emailFrom = Address('not.an.evil.bot@gmail.com')
        self._getIncomingLines(str(emailFrom))
        self._setUpResponder()
        self.responder.incoming.canonicalFromSMTP = smtpFrom
        self.assertFalse(self.responder.runChecks(emailFrom))

    def test_SMTPAutoresponder_runChecks_RCPTTO_From_mismatched_username(self):
        """runChecks() should catch emails where the SMTP 'MAIL FROM:' command
        reported being from an email address and the email's 'From:' header
        reported another email address, even if the only the username part is
        mismatched.
        """
        smtpFrom = 'feidanchaoren0001@gmail.com'
        emailFrom = Address('feidanchaoren0038@gmail.com')
        self._getIncomingLines(str(emailFrom))
        self._setUpResponder()
        self.responder.incoming.canonicalFromSMTP = smtpFrom
        self.assertFalse(self.responder.runChecks(emailFrom))

    def test_SMTPAutoresponder_runChecks_DKIM_dunno(self):
        """runChecks() should catch emails with bad DKIM headers
        (``"X-DKIM-Authentication-Results: dunno"``) for canonical domains
        which we're configured to check DKIM verification results for.
        """
        emailFrom = Address('dkimlikedunno@gmail.com')
        header = "X-DKIM-Authentication-Results: dunno"
        self._getIncomingLines(str(emailFrom))
        self.message.lines.insert(3, header)
        self._setUpResponder()
        self.assertFalse(self.responder.runChecks(emailFrom))

    def test_SMTPAutoresponder_runChecks_DKIM_bad(self):
        """runChecks() should catch emails with bad DKIM headers
        (``"X-DKIM-Authentication-Results: dunno"``) for canonical domains
        which we're configured to check DKIM verification results for.
        """
        emailFrom = Address('dkimlikewat@gmail.com')
        header = "X-DKIM-Authentication-Results: wowie zowie there's a sig here"
        self._getIncomingLines(str(emailFrom))
        self.message.lines.insert(3, header)
        self._setUpResponder()
        self.assertFalse(self.responder.runChecks(emailFrom))

    def test_SMTPAutoresponder_runChecks_blacklisted(self):
        """runChecks() on an blacklisted email address should return False."""
        emailFrom = Address('feidanchaoren0043@gmail.com')
        self._getIncomingLines(str(emailFrom))
        self._setUpResponder()
        self.assertFalse(self.responder.runChecks(emailFrom))



# -*- coding: utf-8 -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
# :copyright: (c) 2014-2015, Isis Lovecruft
#             (c) 2014-2015, The Tor Project, Inc.
# :license: see LICENSE for licensing information

"""Unittests for the :mod:`bridgedb.schedule` module."""

from __future__ import print_function

from twisted.trial import unittest

from bridgedb import schedule


class UnscheduledTests(unittest.TestCase):
    """Tests for :class:`bridgedb.scheduled.Unscheduled`."""

    def setUp(self):
        self.sched = schedule.Unscheduled()

    def test_Unscheduled_init(self):
        """The instance should be an instance of its class."""
        self.assertIsInstance(self.sched, schedule.Unscheduled)

    def test_Unscheduled_providesISchedule(self):
        """Unscheduled should implement the ISchedule interface."""
        schedule.ISchedule.namesAndDescriptions()
        self.assertTrue(schedule.ISchedule.providedBy(self.sched))

    def test_Unscheduled_intervalStart_noargs(self):
        time = self.sched.intervalStart()
        self.assertIsInstance(time, int)
        self.assertEquals(time, -62135596800)

    def test_Unscheduled_getInterval_is_constant(self):
        import time
        now = time.time()

        interval_default = self.sched.getInterval()
        self.assertIsInstance(interval_default, str)

        interval_zero = self.sched.getInterval(0)
        self.assertIsInstance(interval_zero, str)

        interval_now = self.sched.getInterval(now)
        self.assertIsInstance(interval_now, str)

        self.assertEquals(interval_default, interval_zero)
        self.assertEquals(interval_default, interval_now)

    def test_Unscheduled_nextIntervalStarts_noargs(self):
        time = self.sched.nextIntervalStarts()
        self.assertIsInstance(time, int)
        self.assertEquals(time, 253402300799)


class ScheduledIntervalTests(unittest.TestCase):
    """Tests for :class:`bridgedb.scheduled.ScheduledInterval`."""

    def setUp(self):
        import time
        self.now = time.time
        self.sched = schedule.ScheduledInterval

    def test_ScheduledInterval_providesISchedule(self):
        """ScheduledInterval should implement the ISchedule interface."""
        self.assertTrue(schedule.ISchedule.providedBy(self.sched('month', 1)))

    def _check_init(self, sched):
        """The instance should be an instance of its class."""
        self.assertIsInstance(sched, schedule.ScheduledInterval)

    def test_ScheduledInterval_init_month(self):
        self._check_init(self.sched('month', 1))

    def test_ScheduledInterval_init_week(self):
        self._check_init(self.sched('week', 2))

    def test_ScheduledInterval_init_day(self):
        self._check_init(self.sched('days', 5))

    def test_ScheduledInterval_init_hour(self):
        self._check_init(self.sched('hours', 12))

    def test_ScheduledInterval_init_minute(self):
        self._check_init(self.sched('minute', 10))

    def test_ScheduledInterval_init_seconds(self):
        self._check_init(self.sched('seconds', 30))

    def test_ScheduledInterval_init_badIntervalPeriod(self):
        self.assertRaises(schedule.UnknownInterval,
                          self.sched, 'decades', 2)

    def test_ScheduledInterval_init_badIntervalCount(self):
        self.assertRaises(schedule.UnknownInterval,
                          self.sched, 'minutes', 'd20')

    def test_ScheduledInterval_init_negativeIntervalCount(self):
        sched = self.sched('days', -100000)
        self.assertEquals(sched.intervalCount, 1)
        self.assertEquals(sched.intervalPeriod, 'day')

    def test_ScheduledInterval_init_noargs(self):
        """Check that the defaults parameters function as expected."""
        sched = self.sched()
        self.assertEquals(sched.intervalCount, 1)
        self.assertEquals(sched.intervalPeriod, 'hour')

    def _check_intervalStart(self, period='second', count=30, variance=30):
        """Test the ScheduledInterval.intervalStart() method.

        :param str period: The interval type for the period.
        :param int count: The number of **period**s within an interval.
        :param int variance: The amount of variance (in seconds) to tolerate
            between the start of the interval containing now, and now.
        """
        now = int(self.now())
        sched = self.sched(period, count)
        time = sched.intervalStart(now)
        self.assertIsInstance(time, int)
        self.assertApproximates(now, time, variance)

    def test_ScheduledInterval_intervalStart_month(self):
        self._check_intervalStart('month', 1, 31*24*60*60)

    def test_ScheduledInterval_intervalStart_week(self):
        self._check_intervalStart('week', 2, 14*24*60*60)

    def test_ScheduledInterval_intervalStart_day(self):
        self._check_intervalStart('days', 5, 5*24*60*60)

    def test_ScheduledInterval_intervalStart_hour(self):
        self._check_intervalStart('hours', 12, 12*60*60)

    def test_ScheduledInterval_intervalStart_minute(self):
        self._check_intervalStart('minute', 10, 10*60)

    def test_ScheduledInterval_intervalStart_seconds(self):
        self._check_intervalStart('seconds', 30, 30)

    def _check_getInterval(self, period='second', count=30, variance=30):
        """Test the ScheduledInterval.getInterval() method.

        :param str period: The interval type for the period.
        :param int count: The number of **period**s within an interval.
        :param int variance: The amount of variance (in seconds) to tolerate
            between the start of the interval containing now, and now.
        """
        now = int(self.now())
        sched = self.sched(period, count)
        ts = sched.getInterval(now)
        self.assertIsInstance(ts, str)
        secs = [int(x) for x in ts.replace('-', ' ').replace(':', ' ').split()]
        [secs.append(0) for _ in xrange(6-len(secs))]
        secs = schedule.calendar.timegm(secs)
        self.assertApproximates(now, secs, variance)

    def test_ScheduledInterval_getInterval_month(self):
        self._check_getInterval('month', 2, 2*31*24*60*60)

    def test_ScheduledInterval_getInterval_week(self):
        self._check_getInterval('week', 1, 7*24*60*60)

    def test_ScheduledInterval_getInterval_day(self):
        self._check_getInterval('days', 4, 4*24*60*60)

    def test_ScheduledInterval_getInterval_hour(self):
        self._check_getInterval('hours', 23, 23*60*60)

    def test_ScheduledInterval_getInterval_minute(self):
        self._check_getInterval('minutes', 15, 15*60)

    def test_ScheduledInterval_getInterval_seconds(self):
        self._check_getInterval('seconds', 10, 60)

    def _check_nextIntervalStarts(self, period='second', count=30, variance=30):
        """Test the ScheduledInterval.nextIntervalStarts() method.

        :param str period: The interval type for the period.
        :param int count: The number of **period**s within an interval.
        :param int variance: The amount of variance (in seconds) to tolerate
            between the start of the interval containing now, and now.
        """
        now = int(self.now())
        sched = self.sched(period, count)
        time = sched.nextIntervalStarts(now)
        self.assertIsInstance(time, int)
        # (now + variance - time) should be > variance
        self.assertApproximates(now + variance, time, variance)

    def test_ScheduledInterval_nextIntervalStarts_month(self):
        self._check_nextIntervalStarts('month', 2, 2*31*24*60*60)

    def test_ScheduledInterval_nextIntervalStarts_week(self):
        self._check_nextIntervalStarts('week', 1, 7*24*60*60)

    def test_ScheduledInterval_nextIntervalStarts_day(self):
        self._check_nextIntervalStarts('days', 4, 4*24*60*60)

    def test_ScheduledInterval_nextIntervalStarts_hour(self):
        self._check_nextIntervalStarts('hours', 23, 23*60*60)

    def test_ScheduledInterval_nextIntervalStarts_minute(self):
        self._check_nextIntervalStarts('minutes', 15, 15*60)

    def test_ScheduledInterval_nextIntervalStarts_seconds(self):
        self._check_nextIntervalStarts('seconds', 10, 10)



# -*- coding: utf-8 -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: Isis Lovecruft 0xA3ADB67A2CDB8B35 <isis@torproject.org>
# :copyright: (c) 2013-2015, Isis Lovecruft
#             (c) 2007-2015, The Tor Project, Inc.
# :license: 3-Clause BSD, see LICENSE for licensing information

"""Unittests for the :mod:`bridgedb.captcha` module."""


import os
import shutil
import time

from base64 import urlsafe_b64decode

from twisted.trial import unittest

from zope.interface import implementedBy
from zope.interface import providedBy

from bridgedb import captcha
from bridgedb import crypto


class CaptchaTests(unittest.TestCase):
    """Tests for :class:`bridgedb.captcha.Captcha`."""

    def test_implementation(self):
        """Captcha class should implement ICaptcha interface."""
        self.assertTrue(captcha.ICaptcha.implementedBy(captcha.Captcha))

    def test_provider(self):
        """ICaptcha should be provided by instances of Captcha."""
        c = captcha.Captcha()
        self.assertTrue(captcha.ICaptcha.providedBy(c))

    def test_get(self):
        """Captcha.get() should return None."""
        c = captcha.Captcha()
        self.assertIsNone(c.get())


class ReCaptchaTests(unittest.TestCase):
    """Tests for :class:`bridgedb.captcha.ReCaptcha`."""

    def setUp(self):
        self.c = captcha.ReCaptcha('publik', 'sekrit')

    def test_init(self):
        """Check the ReCaptcha class stored the private and public keys."""
        self.assertEquals(self.c.secretKey, 'sekrit')
        self.assertEquals(self.c.publicKey, 'publik')

    def test_get(self):
        """Test get() method."""

        # Force urllib2 to do anything less idiotic than the defaults:
        envkey = 'HTTPS_PROXY'
        oldkey = None
        if os.environ.has_key(envkey):
            oldkey = os.environ[envkey]
        os.environ[envkey] = '127.0.0.1:9150'
        # This stupid thing searches the environment for ``<protocol>_PROXY``
        # variables, hence the above 'HTTPS_PROXY' env setting:
        proxy = captcha.urllib2.ProxyHandler()
        opener = captcha.urllib2.build_opener(proxy)
        captcha.urllib2.install_opener(opener)

        try:
            # There isn't really a reliable way to test this function! :(
            self.c.get()
        except Exception as error:
            reason  = "ReCaptcha.get() test requires an active network "
            reason += "connection.\nThis test failed with: %s" % error
            raise unittest.SkipTest(reason)
        else:
            self.assertIsInstance(self.c.image, basestring)
            self.assertIsInstance(self.c.challenge, basestring)
        finally:
            # Replace the original environment variable if there was one:
            if oldkey:
                os.environ[envkey] = oldkey
            else:
                os.environ.pop(envkey)

    def test_get_noKeys(self):
        """ReCaptcha.get() without API keys should fail."""
        c = captcha.ReCaptcha()
        self.assertRaises(captcha.CaptchaKeyError, c.get)


class GimpCaptchaTests(unittest.TestCase):
    """Tests for :class:`bridgedb.captcha.GimpCaptcha`."""

    def setUp(self):
        here             = os.getcwd()
        self.topDir      = here.rstrip('_trial_temp')
        self.cacheDir    = os.path.join(self.topDir, 'captchas')
        self.badCacheDir = os.path.join(here, 'capt')

        # Get keys for testing or create them:
        self.sekrit, self.publik = crypto.getRSAKey('test_gimpCaptcha_RSAkey')
        self.hmacKey = crypto.getKey('test_gimpCaptcha_HMACkey')

    def test_init_noSecretKey(self):
        """Calling GimpCaptcha.__init__() without a secret key parameter should raise
        a CaptchaKeyError.
        """
        self.assertRaises(captcha.CaptchaKeyError, captcha.GimpCaptcha,
                          self.publik, None, self.hmacKey, self.cacheDir)

    def test_init_noPublicKey(self):
        """__init__() without publicKey should raise a CaptchaKeyError."""
        self.assertRaises(captcha.CaptchaKeyError, captcha.GimpCaptcha,
                          None, self.sekrit, self.hmacKey, self.cacheDir)

    def test_init_noHMACKey(self):
        """__init__() without hmacKey should raise a CaptchaKeyError."""
        self.assertRaises(captcha.CaptchaKeyError, captcha.GimpCaptcha,
                          self.publik, self.sekrit, None, self.cacheDir)

    def test_init_noCacheDir(self):
        """__init__() without cacheDir should raise a CaptchaKeyError."""
        self.assertRaises(captcha.GimpCaptchaError, captcha.GimpCaptcha,
                          self.publik, self.sekrit, self.hmacKey, None)

    def test_init_badCacheDir(self):
        """GimpCaptcha with bad cacheDir should raise GimpCaptchaError."""
        self.assertRaises(captcha.GimpCaptchaError, captcha.GimpCaptcha,
                          self.publik, self.sekrit, self.hmacKey,
                          self.cacheDir.rstrip('chas'))

    def test_init(self):
        """Test that __init__ correctly initialised all the values."""
        c = captcha.GimpCaptcha(self.publik, self.sekrit, self.hmacKey,
                                self.cacheDir)
        self.assertIsNone(c.answer)
        self.assertIsNone(c.image)
        self.assertIsNone(c.challenge)

    def test_createChallenge(self):
        """createChallenge() should return the encrypted CAPTCHA answer."""
        c = captcha.GimpCaptcha(self.publik, self.sekrit, self.hmacKey,
                                self.cacheDir)
        challenge = c.createChallenge('w00t')
        self.assertIsInstance(challenge, basestring)

    def test_createChallenge_base64(self):
        """createChallenge() return value should be urlsafe base64-encoded."""
        c = captcha.GimpCaptcha(self.publik, self.sekrit, self.hmacKey,
                                self.cacheDir)
        challenge = c.createChallenge('w00t')
        decoded = urlsafe_b64decode(challenge)
        self.assertTrue(decoded)

    def test_createChallenge_hmacValid(self):
        """The HMAC in createChallenge() return value should be valid."""
        c = captcha.GimpCaptcha(self.publik, self.sekrit, self.hmacKey,
                                self.cacheDir)
        challenge = c.createChallenge('ShouldHaveAValidHMAC')
        decoded = urlsafe_b64decode(challenge)
        hmac = decoded[:20]
        orig = decoded[20:]
        correctHMAC = crypto.getHMAC(self.hmacKey, orig)
        self.assertEquals(hmac, correctHMAC)

    def test_createChallenge_decryptedAnswerMatches(self):
        """The HMAC in createChallenge() return value should be valid."""
        c = captcha.GimpCaptcha(self.publik, self.sekrit, self.hmacKey,
                                self.cacheDir)
        challenge = c.createChallenge('ThisAnswerShouldDecryptToThis')
        decoded = urlsafe_b64decode(challenge)
        hmac = decoded[:20]
        orig = decoded[20:]
        correctHMAC = crypto.getHMAC(self.hmacKey, orig)
        self.assertEqual(hmac, correctHMAC)

        decrypted = self.sekrit.decrypt(orig)
        timestamp = int(decrypted[:12].lstrip('0'))
        # The timestamp should be within 30 seconds of right now.
        self.assertApproximates(timestamp, int(time.time()), 30)
        self.assertEqual('ThisAnswerShouldDecryptToThis', decrypted[12:])

    def test_get(self):
        """GimpCaptcha.get() should return image and challenge strings."""
        c = captcha.GimpCaptcha(self.publik, self.sekrit, self.hmacKey,
                                self.cacheDir)
        image, challenge = c.get()
        self.assertIsInstance(image, basestring)
        self.assertIsInstance(challenge, basestring)

    def test_get_emptyCacheDir(self):
        """An empty cacheDir should raise GimpCaptchaError."""
        os.makedirs(self.badCacheDir)
        c = captcha.GimpCaptcha(self.publik, self.sekrit, self.hmacKey,
                                self.badCacheDir)
        self.assertRaises(captcha.GimpCaptchaError, c.get)
        shutil.rmtree(self.badCacheDir)

    def test_get_unreadableCaptchaFile(self):
        """An unreadable CAPTCHA file should raise GimpCaptchaError."""
        os.makedirs(self.badCacheDir)
        badFile = os.path.join(self.badCacheDir, 'uNr34dA81e.jpg')
        with open(badFile, 'w') as fh:
            fh.write(' ')
            fh.flush()
        os.chmod(badFile, 0266)

        c = captcha.GimpCaptcha(self.publik, self.sekrit, self.hmacKey,
                                self.badCacheDir)
        # This should hit the second `except:` clause in get():
        self.assertRaises(captcha.GimpCaptchaError, c.get)
        shutil.rmtree(self.badCacheDir)

    def test_check(self):
        """A correct answer and valid challenge should return True."""
        c = captcha.GimpCaptcha(self.publik, self.sekrit, self.hmacKey,
                                self.cacheDir)
        image, challenge = c.get()
        self.assertEquals(
            c.check(challenge, c.answer, c.secretKey, c.hmacKey),
            True)

    def test_check_blankAnswer(self):
        """A blank answer and valid challenge should return False."""
        c = captcha.GimpCaptcha(self.publik, self.sekrit, self.hmacKey,
                                self.cacheDir)
        image, challenge = c.get()
        self.assertEquals(
            c.check(challenge, None, c.secretKey, c.hmacKey),
            False)

    def test_check_nonBase64(self):
        """Valid answer and challenge with invalid base64 returns False."""
        c = captcha.GimpCaptcha(self.publik, self.sekrit, self.hmacKey,
                                self.cacheDir)
        image, challenge = c.get()
        challengeBadB64 = challenge.rstrip('==') + "\x42\x42\x42"
        self.assertEquals(
            c.check(challenge, c.answer, c.secretKey, c.hmacKey),
            True)
        self.assertEquals(
            c.check(challengeBadB64, c.answer, c.secretKey, c.hmacKey),
            False)

    def test_check_caseInsensitive_lowercase(self):
        """A correct answer in lowercase characters should return True."""
        c = captcha.GimpCaptcha(self.publik, self.sekrit, self.hmacKey,
                                self.cacheDir)
        image, challenge = c.get()
        solution = c.answer.lower()
        self.assertEquals(
            c.check(challenge, solution, c.secretKey, c.hmacKey),
            True)

    def test_check_caseInsensitive_uppercase(self):
        """A correct answer in uppercase characters should return True."""
        c = captcha.GimpCaptcha(self.publik, self.sekrit, self.hmacKey,
                                self.cacheDir)
        image, challenge = c.get()
        solution = c.answer.upper()
        self.assertEquals(
            c.check(challenge, solution, c.secretKey, c.hmacKey),
            True)

    def test_check_encoding_utf8(self):
        """A correct answer in utf-8 lowercase should return True."""
        c = captcha.GimpCaptcha(self.publik, self.sekrit, self.hmacKey,
                                self.cacheDir)
        image, challenge = c.get()
        solution = c.answer.encode('utf8')
        self.assertEquals(
            c.check(challenge, solution, c.secretKey, c.hmacKey),
            True)

    def test_check_encoding_ascii(self):
        """A correct answer in utf-8 lowercase should return True."""
        c = captcha.GimpCaptcha(self.publik, self.sekrit, self.hmacKey,
                                self.cacheDir)
        image, challenge = c.get()
        solution = c.answer.encode('ascii')
        self.assertEquals(
            c.check(challenge, solution, c.secretKey, c.hmacKey),
            True)

    def test_check_encoding_unicode(self):
        """A correct answer in utf-8 lowercase should return True."""
        c = captcha.GimpCaptcha(self.publik, self.sekrit, self.hmacKey,
                                self.cacheDir)
        image, challenge = c.get()
        solution = unicode(c.answer)
        self.assertEquals(
            c.check(challenge, solution, c.secretKey, c.hmacKey),
            True)

    def test_check_missingHMACbytes(self):
        """A challenge that is missing part of the HMAC should return False."""
        c = captcha.GimpCaptcha(self.publik, self.sekrit, self.hmacKey,
                                self.cacheDir)
        image, challenge = c.get()
        challengeBadHMAC = challenge[:10] + challenge[20:]
        self.assertEquals(
            c.check(challengeBadHMAC, c.answer, c.secretKey, c.hmacKey),
            False)

    def test_check_missingAnswerbytes(self):
        """Partial encrypted answers in challenges should return False."""
        c = captcha.GimpCaptcha(self.publik, self.sekrit, self.hmacKey,
                                self.cacheDir)
        image, challenge = c.get()
        challengeBadOrig = challenge[:20] + challenge[30:]
        self.assertEquals(
            c.check(challengeBadOrig, c.answer, c.secretKey, c.hmacKey),
            False)

    def test_check_badHMACkey(self):
        """A challenge with a bad HMAC key should return False."""
        hmacKeyBad = crypto.getKey('test_gimpCaptcha_badHMACkey')
        c = captcha.GimpCaptcha(self.publik, self.sekrit, self.hmacKey,
                                self.cacheDir)
        image, challenge = c.get()
        self.assertEquals(
            c.check(challenge, c.answer, c.secretKey, hmacKeyBad),
            False)

    def test_check_badRSAkey(self):
        """A challenge with a bad RSA secret key should return False."""
        secretKeyBad, publicKeyBad = crypto.getRSAKey('test_gimpCaptcha_badRSAkey')
        c = captcha.GimpCaptcha(self.publik, self.sekrit, self.hmacKey,
                                self.cacheDir)
        image, challenge = c.get()
        self.assertEquals(
            c.check(challenge, c.answer, secretKeyBad, c.hmacKey),
            False)



# -*- coding: utf-8 -*-
#
# This file is part of BridgeDB, a Tor bridge distribution system.
#
# :authors: please see the AUTHORS file for attributions
# :copyright: (c) 2013-2015, Isis Lovecruft
#             (c) 2007-2015, The Tor Project, Inc.
# :license: see LICENSE for licensing information

"""Tests for :mod:`bridgedb.configure`."""

from __future__ import print_function

import os

from twisted.trial import unittest

from bridgedb import configure


class ConfigureTests(unittest.TestCase):
    """Tests for miscelaneous functions in :mod:`bridgedb.configure`."""

    def setUp(self):
        """Find the config file in the top directory of this repo."""
        here = os.getcwd()
        topdir = here.rstrip('_trial_temp')
        self.configFilename = os.path.join(topdir, 'bridgedb.conf')

    def test_loadConfig_with_file(self):
        """We should be able to load and parse the standard ``bridgedb.conf``
        file from the top directory of this repository.
        """
        config = configure.loadConfig(self.configFilename)
        self.assertTrue(config)

    def test_loadConfig_with_file_and_class(self):
        """We should be able to reload and parse the ``bridgedb.conf``
        file, if we have a config class as well.
        """
        config = configure.loadConfig(self.configFilename)
        newConfig = configure.loadConfig(self.configFilename, configCls=config)
        self.assertTrue(newConfig)

    def test_loadConfig_with_class(self):
        """We should be able to recreate a config, given its class."""
        config = configure.loadConfig(self.configFilename)
        newConfig = configure.loadConfig(configCls=config)
        self.assertTrue(newConfig)

    def test_loadConfig_set_EXTRA_INFO_FILES_when_None(self):
        """If certain options, like the ``EXTRA_INFO_FILES`` option in the
        config file weren't set, they should be made into lists so that our
        parsers don't choke on them later.
        """
        config = configure.loadConfig(self.configFilename)
        setattr(config, "EXTRA_INFO_FILES", None)
        self.assertTrue(config.EXTRA_INFO_FILES is None)
        newConfig = configure.loadConfig(configCls=config)
        self.assertIsInstance(newConfig.EXTRA_INFO_FILES, list)

    def test_loadConfig_returns_Conf(self):
        """After loading and parsing the ``bridgedb.conf`` file, we should have
        a :class:`bridgedb.configure.Conf`.
        """
        config = configure.loadConfig(self.configFilename)
        self.assertIsInstance(config, configure.Conf)
